@@ -9,11 +9,10 @@ import           Matterhorn.Prelude
 
 import           Brick
 import           Control.Monad.Trans.Except ( runExceptT )
-import qualified Data.HashMap.Strict as HM
 import qualified Graphics.Vty as Vty
+import qualified Graphics.Vty.CrossPlatform as Vty
 import           Text.Aspell ( stopAspell )
 import           GHC.Conc (getNumProcessors, setNumCapabilities)
-import           System.Posix.IO ( stdInput )
 
 import           Network.Mattermost
 
@@ -30,34 +29,45 @@ import           Matterhorn.Types
 
 
 app :: App ChatState MHEvent Name
-app = App
-  { appDraw         = draw
-  , appChooseCursor = \s cs -> case s^.csCurrentTeam.tsMode of
-      Main                          -> showFirstCursor s cs
-      ChannelSelect                 -> showFirstCursor s cs
-      UserListOverlay               -> showFirstCursor s cs
-      ReactionEmojiListOverlay      -> showFirstCursor s cs
-      ChannelListOverlay            -> showFirstCursor s cs
-      ManageAttachmentsBrowseFiles  -> showFirstCursor s cs
-      ThemeListOverlay              -> showFirstCursor s cs
-      ChannelTopicWindow            -> let tId = s^.csCurrentTeamId
-                                       in showCursorNamed (ChannelTopicEditor tId) cs
-      SaveAttachmentWindow _        -> let tId = s^.csCurrentTeamId
-                                       in showCursorNamed (AttachmentPathEditor tId) cs
-      LeaveChannelConfirm           -> Nothing
-      DeleteChannelConfirm          -> Nothing
-      MessageSelect                 -> Nothing
-      MessageSelectDeleteConfirm    -> Nothing
-      PostListOverlay _             -> Nothing
-      ManageAttachments             -> Nothing
-      ViewMessage                   -> Nothing
-      ShowHelp _ _                  -> Nothing
-      UrlSelect                     -> Nothing
-      EditNotifyPrefs               -> Nothing
-  , appHandleEvent  = Events.onEvent
-  , appStartEvent   = return
-  , appAttrMap      = (^.csResources.crTheme)
-  }
+app =
+    App { appDraw         = draw
+        , appHandleEvent  = Events.onEvent
+        , appAttrMap      = (^.csResources.crTheme)
+        , appChooseCursor = \s cs -> do
+            tId <- s^.csCurrentTeamId
+            cursorByMode cs s tId (teamMode $ s^.csTeam(tId))
+        , appStartEvent = do
+            vty <- getVtyHandle
+            (w, h) <- liftIO $ Vty.displayBounds $ Vty.outputIface vty
+            runMHEvent $ Events.setWindowSize w h
+        }
+
+cursorByMode :: [CursorLocation Name] -> ChatState -> TeamId -> Mode -> Maybe (CursorLocation Name)
+cursorByMode cs s tId mode =
+    case mode of
+        Main -> case s^.csTeam(tId).tsMessageInterfaceFocus of
+            FocusCurrentChannel -> do
+                cId <- s^.csCurrentChannelId(tId)
+                mi <- s^?maybeChannelMessageInterface(cId)
+                cur <- messageInterfaceCursor mi
+                showCursorNamed cur cs
+            FocusThread -> do
+                ti <- s^.csTeam(tId).tsThreadInterface
+                cur <- messageInterfaceCursor ti
+                showCursorNamed cur cs
+        LeaveChannelConfirm           -> Nothing
+        DeleteChannelConfirm          -> Nothing
+        MessageSelectDeleteConfirm {} -> Nothing
+        (PostListWindow {})           -> Nothing
+        ViewMessage                   -> Nothing
+        (ShowHelp {})                 -> Nothing
+        EditNotifyPrefs               -> Nothing
+        ChannelSelect                 -> showFirstCursor s cs
+        UserListWindow                -> showFirstCursor s cs
+        ReactionEmojiListWindow       -> showFirstCursor s cs
+        ChannelListWindow             -> showFirstCursor s cs
+        ThemeListWindow               -> showFirstCursor s cs
+        ChannelTopicWindow            -> showCursorNamed (ChannelTopicEditor tId) cs
 
 applicationMaxCPUs :: Int
 applicationMaxCPUs = 2
@@ -77,12 +87,7 @@ runMatterhorn opts config = do
     setupCpuUsage config
 
     let mkVty = do
-          mEraseChar <- Vty.getTtyEraseChar stdInput
-          let addEraseChar cfg = case mEraseChar of
-                  Nothing -> cfg
-                  Just ch -> cfg { Vty.inputMap = (Nothing, [ch], Vty.EvKey Vty.KBS []) : Vty.inputMap cfg }
-
-          vty <- Vty.mkVty $ addEraseChar Vty.defaultConfig
+          vty <- Vty.mkVty Vty.defaultConfig
           let output = Vty.outputIface vty
           Vty.setMode output Vty.BracketedPaste True
           Vty.setMode output Vty.Hyperlink $ configHyperlinkingMode config
@@ -92,10 +97,9 @@ runMatterhorn opts config = do
     (st, vty) <- setupState mkVty (optLogLocation opts) config
     finalSt <- customMain vty mkVty (Just $ st^.csResources.crEventQueue) app st
 
-    forM_ (HM.elems $ finalSt^.csTeams) $ \ts ->
-        case ts^.tsEditState.cedSpellChecker of
-            Nothing -> return ()
-            Just (s, _) -> stopAspell s
+    case st^.csResources.crSpellChecker of
+        Nothing -> return ()
+        Just s -> stopAspell s
 
     return finalSt
 
