@@ -18,12 +18,13 @@ import           Matterhorn.Themes
 buttonWidth :: Int
 buttonWidth = 10
 
-drawButton :: (Eq n) => FocusRing n -> n -> T.Text -> Widget n
+drawButton :: (Eq n, Ord n) => FocusRing n -> n -> T.Text -> Widget n
 drawButton f n label =
     let attr = if focusGetCurrent f == Just n
                then buttonFocusedAttr
                else buttonAttr
     in withDefAttr attr $
+       clickable n $
        hLimit buttonWidth $
        hCenter $
        txt label
