@@ -16,14 +16,16 @@ import           Control.Arrow ( (>>>) )
 import qualified Data.Text as T
 import           Data.Text.Zipper ( insertChar, gotoEOL )
 
+import           Network.Mattermost.Types ( TeamId )
+
 import           Matterhorn.Types
 import           Matterhorn.Draw.Buttons
 import           Matterhorn.Draw.RichText
 import           Matterhorn.Themes
 
 
-drawChannelTopicWindow :: ChatState -> Widget Name
-drawChannelTopicWindow st =
+drawChannelTopicWindow :: ChatState -> TeamId -> Widget Name
+drawChannelTopicWindow st tId =
     centerLayer $
     hLimit maxWindowWidth $
     joinBorders $
@@ -35,6 +37,9 @@ drawChannelTopicWindow st =
            viewport (ChannelTopicEditorPreview tId) Vertical $
            renderText' (Just baseUrl) "" hSet Nothing topicTxtWithCursor
          , hBorder
+         , hCenter $
+           renderText' Nothing "" hSet Nothing
+           "Press `Tab` to change focus to buttons."
          , hBox [ padRight Max $
                   padLeft (Pad 1) $
                   drawButton foc (ChannelTopicSaveButton tId) "Save"
@@ -44,13 +49,12 @@ drawChannelTopicWindow st =
          ]
     where
         baseUrl = serverBaseUrl st tId
-        tId = st^.csCurrentTeamId
         editorHeight = 5
         previewHeight = 5
         maxWindowWidth = 70
-        foc = st^.csCurrentTeam.tsChannelTopicDialog.channelTopicDialogFocus
-        ed = st^.csCurrentTeam.tsChannelTopicDialog.channelTopicDialogEditor
-        hSet = getHighlightSet st
+        foc = st^.csTeam(tId).tsChannelTopicDialog.channelTopicDialogFocus
+        ed = st^.csTeam(tId).tsChannelTopicDialog.channelTopicDialogEditor
+        hSet = getHighlightSet st tId
         topicTxtWithCursor = T.unlines $
                              getEditContents $
                              applyEdit (gotoEOL >>> insertChar cursorSentinel) ed
