@@ -9,6 +9,7 @@ module Matterhorn.Draw.Messages
   , unsafeRenderMessageSelection
   , renderLastMessages
   , addEllipsis
+  , mkClickableInline
   )
 where
 
@@ -25,6 +26,7 @@ import           Data.Sequence ( ViewL(..)
                                , viewr)
 import qualified Data.Set as Set
 import qualified Data.Text as T
+import           Data.Maybe ( fromJust )
 import qualified Graphics.Vty as V
 import           Lens.Micro.Platform ( (.~), to )
 import           Network.Mattermost.Lenses ( postEditAtL, postCreateAtL )
@@ -67,6 +69,8 @@ data MessageData =
                 -- ^ The thread state of this message.
                 , mdRenderReplyParent :: Bool
                 -- ^ Whether to render the parent message.
+                , mdRenderReplyIndent :: Bool
+                -- ^ Whether to render reply indent decorations
                 , mdHighlightSet :: HighlightSet
                 -- ^ The highlight set to use to highlight usernames,
                 -- channel names, etc.
@@ -91,6 +95,9 @@ data MessageData =
                 , mdWrapNonhighlightedCodeBlocks :: Bool
                 -- ^ Whether to wrap text in non-highlighted code
                 -- blocks.
+                , mdClickableNameTag :: Name
+                -- ^ Used to namespace clickable extents produced by
+                -- rendering this message
                 }
 
 maxMessageHeight :: Int
@@ -117,6 +124,8 @@ renderSingleMessage :: ChatState
                     -> HighlightSet
                     -- ^ The highlight set to use when rendering this
                     -- message
+                    -> Bool
+                    -- ^ Whether to render reply indentations
                     -> Maybe ServerTime
                     -- ^ This specifies an "indicator boundary". Showing
                     -- various indicators (e.g. "edited") is not
@@ -126,9 +135,13 @@ renderSingleMessage :: ChatState
                     -- ^ The message to render
                     -> ThreadState
                     -- ^ The thread state in which to render the message
+                    -> Name
+                    -- ^ Clickable name tag
                     -> Widget Name
-renderSingleMessage st hs ind m threadState =
-  renderChatMessage st hs ind threadState (withBrackets . renderTime st . withServerTime) m
+renderSingleMessage st hs renderReplyIndent ind m threadState tag =
+  renderChatMessage st hs ind threadState tag
+                    (withBrackets . renderTime st . withServerTime)
+                    renderReplyIndent m
 
 renderChatMessage :: ChatState
                   -- ^ The application state
@@ -142,12 +155,17 @@ renderChatMessage :: ChatState
                   -- value.
                   -> ThreadState
                   -- ^ The thread state in which to render the message
+                  -> Name
+                  -- ^ The UI region in which the message is being
+                  -- rendered (for tagging clickable extents)
                   -> (ServerTime -> Widget Name)
                   -- ^ A function to render server times
+                  -> Bool
+                  -- ^ Whether to render reply indentations
                   -> Message
                   -- ^ The message to render
                   -> Widget Name
-renderChatMessage st hs ind threadState renderTimeFunc msg =
+renderChatMessage st hs ind threadState clickableNameTag renderTimeFunc renderReplyIndent msg =
     let showOlderEdits = configShowOlderEdits config
         showTimestamp = configShowMessageTimestamps config
         config = st^.csResources.crConfiguration
@@ -163,6 +181,7 @@ renderChatMessage st hs ind threadState renderTimeFunc msg =
               , mdHighlightSet      = hs
               , mdShowOlderEdits    = showOlderEdits
               , mdRenderReplyParent = True
+              , mdRenderReplyIndent = renderReplyIndent
               , mdIndentBlocks      = True
               , mdThreadState       = threadState
               , mdShowReactions     = True
@@ -171,6 +190,7 @@ renderChatMessage st hs ind threadState renderTimeFunc msg =
               , mdMyUserId          = userId $ myUser st
               , mdWrapNonhighlightedCodeBlocks = True
               , mdTruncateVerbatimBlocks = st^.csVerbatimTruncateSetting
+              , mdClickableNameTag  = clickableNameTag
               }
         fullMsg =
           case msg^.mUser of
@@ -210,23 +230,25 @@ unsafeRenderMessageSelection :: (SeqDirection dir1, SeqDirection dir2)
                                 )
                              -- ^ The message to render, the messages
                              -- before it, and after it, respectively
-                             -> (Message -> ThreadState -> Widget Name)
+                             -> (Message -> ThreadState -> Name -> Widget Name)
                              -- ^ A per-message rendering function to
                              -- use
+                             -> Name
+                             -- ^ Clickable name tag
                              -> Widget Name
-unsafeRenderMessageSelection ((curMsg, curThreadState), (before, after)) doMsgRender =
+unsafeRenderMessageSelection ((curMsg, curThreadState), (before, after)) doMsgRender tag =
   Widget Greedy Greedy $ do
     ctx <- getContext
     curMsgResult <- withReaderT relaxHeight $ render $
-                    forceAttr messageSelectAttr $
-                    padRight Max $ doMsgRender curMsg curThreadState
+                    forceAttrAllowStyle messageSelectAttr $
+                    padRight Max $ doMsgRender curMsg curThreadState tag
 
     let targetHeight = ctx^.availHeightL
         upperHeight = targetHeight `div` 2
         lowerHeight = targetHeight - upperHeight
 
-    lowerHalfResults <- renderMessageSeq targetHeight (render1 doMsgRender) vLimit after
-    upperHalfResults <- renderMessageSeq targetHeight (render1 doMsgRender) cropTopTo before
+    lowerHalfResults <- renderMessageSeq targetHeight (render1 doMsgRender) vLimit tag after
+    upperHalfResults <- renderMessageSeq targetHeight (render1 doMsgRender) cropTopTo tag before
 
     let upperHalfResultsHeight = sum $ (V.imageHeight . image) <$> upperHalfResults
         lowerHalfResultsHeight = sum $ (V.imageHeight . image) <$> lowerHalfResults
@@ -256,31 +278,34 @@ unsafeRenderMessageSelection ((curMsg, curThreadState), (before, after)) doMsgRe
 
 renderMessageSeq :: (SeqDirection dir)
                  => Int
-                 -> (Message -> ThreadState -> Widget Name)
+                 -> (Message -> ThreadState -> Name -> Widget Name)
                  -> (Int -> Widget Name -> Widget Name)
+                 -> Name
                  -> DirectionalSeq dir (Message, ThreadState)
                  -> RenderM Name [Result Name]
-renderMessageSeq remainingHeight renderFunc limitFunc ms
+renderMessageSeq remainingHeight renderFunc limitFunc tag ms
     | messagesLength ms == 0 = return []
     | otherwise = do
-        let Just (m, threadState) = messagesHead ms
+        let (m, threadState) = fromJust $ messagesHead ms
             maybeCache = case m^.mMessageId of
                 Nothing -> id
                 Just i -> cached (RenderedMessage i)
-        result <- render $ limitFunc remainingHeight $ maybeCache $ renderFunc m threadState
-        rest <- renderMessageSeq (remainingHeight - (V.imageHeight $ result^.imageL)) renderFunc limitFunc (messagesDrop 1 ms)
+        result <- render $ limitFunc remainingHeight $ maybeCache $ renderFunc m threadState tag
+        rest <- renderMessageSeq (remainingHeight - (V.imageHeight $ result^.imageL)) renderFunc limitFunc tag (messagesDrop 1 ms)
         return $ result : rest
 
 renderLastMessages :: ChatState
                    -> HighlightSet
                    -> Maybe ServerTime
+                   -> Bool
+                   -> Name
                    -> DirectionalSeq Retrograde (Message, ThreadState)
                    -> Widget Name
-renderLastMessages st hs editCutoff msgs =
+renderLastMessages st hs editCutoff renderReplyIndent tag msgs =
     Widget Greedy Greedy $ do
         ctx <- getContext
         let targetHeight = ctx^.availHeightL
-            doMsgRender = renderSingleMessage st hs editCutoff
+            doMsgRender = renderSingleMessage st hs renderReplyIndent editCutoff
 
             newMessagesTransitions = filterMessages (isNewMessagesTransition . fst) msgs
             newMessageTransition = fst <$> (listToMaybe $ F.toList newMessagesTransitions)
@@ -290,10 +315,10 @@ renderLastMessages st hs editCutoff msgs =
             go :: Int -> DirectionalSeq Retrograde (Message, ThreadState) -> RenderM Name [Result Name]
             go _ ms | messagesLength ms == 0 = return []
             go remainingHeight ms = do
-                let Just (m, threadState) = messagesHead ms
+                let (m, threadState) = fromJust $ messagesHead ms
                     newMessagesAbove = maybe False (isBelow m) newMessageTransition
 
-                result <- render $ render1 doMsgRender m threadState
+                result <- render $ render1 doMsgRender m threadState tag
 
                 croppedResult <- render $ cropTopTo remainingHeight $ resultToWidget result
 
@@ -322,20 +347,21 @@ renderLastMessages st hs editCutoff msgs =
         results <- go targetHeight msgs
         render $ vBox $ resultToWidget <$> reverse results
 
-relaxHeight :: Context -> Context
+relaxHeight :: Context n -> Context n
 relaxHeight c = c & availHeightL .~ (max maxMessageHeight (c^.availHeightL))
 
-render1 :: (Message -> ThreadState -> Widget Name)
+render1 :: (Message -> ThreadState -> Name -> Widget Name)
         -> Message
         -> ThreadState
+        -> Name
         -> Widget Name
-render1 doMsgRender msg threadState = case msg^.mDeleted of
+render1 doMsgRender msg threadState tag = case msg^.mDeleted of
     True -> emptyWidget
     False ->
         Widget Greedy Fixed $ do
             withReaderT relaxHeight $
                 render $ padRight Max $
-                doMsgRender msg threadState
+                doMsgRender msg threadState tag
 
 -- | This performs rendering of the specified message according to
 -- settings in MessageData.
@@ -357,7 +383,7 @@ renderMessage md@MessageData { mdMessage = msg, .. } =
             Nothing -> id
             -- We use the index (-1) since indexes for clickable
             -- usernames elsewhere in this message start at 0.
-            Just i -> clickable (ClickableUsernameInMessage i (-1) un)
+            Just i -> clickable (ClickableUsername (Just i) mdClickableNameTag (-1) un)
 
         nameElems = case msgUsr of
           Just un
@@ -397,16 +423,21 @@ renderMessage md@MessageData { mdMessage = msg, .. } =
 
         replyIndent = Widget Fixed Fixed $ do
             ctx <- getContext
-            -- NB: The amount subtracted here must be the total padding
-            -- added below (pad 1 + vBorder)
-            w <- render $ hLimit (ctx^.availWidthL - 2) msgWidget
-            render $ vLimit (V.imageHeight $ w^.imageL) $
-                padRight (Pad 1) vBorder <+> resultToWidget w
+            w <- case mdRenderReplyIndent of
+                True -> do
+                    -- NB: The amount subtracted here must be the total
+                    -- padding added below (pad 1 + vBorder)
+                    w <- render $ hLimit (ctx^.availWidthL - 2) msgWidget
+                    return $ vLimit (V.imageHeight $ w^.imageL) $
+                        padRight (Pad 1) vBorder <+> resultToWidget w
+                False ->
+                    return msgWidget
+            render w
 
         msgAtch = if S.null (msg^.mAttachments)
           then Nothing
           else Just $ withDefAttr clientMessageAttr $ vBox
-                 [ padLeft (Pad 2) $ clickable (ClickableAttachment (a^.attachmentFileId)) $
+                 [ padLeft (Pad 2) $ clickable (ClickableAttachmentInMessage mdClickableNameTag (a^.attachmentFileId)) $
                                      txt ("[attached: `" <> a^.attachmentName <> "`]")
                  | a <- toList (msg^.mAttachments)
                  ]
@@ -417,7 +448,8 @@ renderMessage md@MessageData { mdMessage = msg, .. } =
                 InThreadShowParent -> p <=> replyIndent
                 InThread           -> replyIndent
 
-    in if not mdRenderReplyParent
+    in freezeBorders $
+       if not mdRenderReplyParent
        then msgWidget
        else case msg^.mInReplyToMsg of
           NotAReply -> msgWidget
@@ -433,7 +465,7 @@ renderMessage md@MessageData { mdMessage = msg, .. } =
                             , mdRenderReplyParent = False
                             , mdIndentBlocks      = False
                             }
-                      in withParent (addEllipsis $ forceAttr replyParentAttr parentMsg)
+                      in withParent (addEllipsis $ forceAttrAllowStyle replyParentAttr parentMsg)
 
     where
         renderBlocks :: HighlightSet -> Maybe Int -> [Widget Name] -> Seq Block
@@ -453,7 +485,7 @@ renderMessage md@MessageData { mdMessage = msg, .. } =
                          , hBox [txt "  ", renderRichText mdMyUsername hs ((subtract 2) <$> w)
                                                  mdWrapNonhighlightedCodeBlocks
                                                  mdTruncateVerbatimBlocks
-                                                 (Just mkClickableNames) (Blocks bs)]
+                                                 (Just (mkClickableInline (msg^.mMessageId) mdClickableNameTag)) (Blocks bs)]
                          ]
                else nameNextToMessage hs w nameElems bs
 
@@ -465,20 +497,20 @@ renderMessage md@MessageData { mdMessage = msg, .. } =
                               , renderRichText mdMyUsername hs newW
                                   mdWrapNonhighlightedCodeBlocks
                                   mdTruncateVerbatimBlocks
-                                  (Just mkClickableNames) (Blocks bs)
+                                  (Just (mkClickableInline (msg^.mMessageId) mdClickableNameTag)) (Blocks bs)
                               ]
 
         isBreak i = i `elem` [ELineBreak, ESoftBreak]
 
-        mkClickableNames i (EHyperlink u _) =
-            case msg^.mMessageId of
-                Just mId -> Just $ ClickableURLInMessage mId i $ LinkURL u
-                Nothing -> Nothing
-        mkClickableNames i (EUser name) =
-            case msg^.mMessageId of
-                Just mId -> Just $ ClickableUsernameInMessage mId i name
-                Nothing -> Nothing
-        mkClickableNames _ _ = Nothing
+mkClickableInline :: Maybe MessageId -> Name -> Int -> Inline -> Maybe Name
+mkClickableInline mmId scope i (EHyperlink u _) = do
+    return $ ClickableURL mmId scope i $ LinkURL u
+mkClickableInline mmId scope i (EUser name) =
+    return $ ClickableUsername mmId scope i name
+mkClickableInline mmId scope i (EPermalink teamUrlName pId _) =
+    return $ ClickableURL mmId scope i $ LinkPermalink teamUrlName pId
+mkClickableInline _ _ _ _ =
+    Nothing
 
 messageReactions :: MessageData -> Maybe (Widget Name)
 messageReactions MessageData { mdMessage = msg, .. } =
@@ -500,7 +532,7 @@ messageReactions MessageData { mdMessage = msg, .. } =
              hasAnyReactions = not $ null nonEmptyReactions
              makeName e us = do
                  pid <- postId <$> msg^.mOriginalPost
-                 Just $ ClickableReactionInMessage pid e us
+                 Just $ ClickableReaction pid mdClickableNameTag e us
              reactionWidget = Widget Fixed Fixed $ do
                  ctx <- getContext
                  let lineW = ctx^.availWidthL
