@@ -11,15 +11,17 @@ import Brick.Widgets.Border
 import Brick.Widgets.Center
 import Brick.Forms (renderForm)
 import Data.List (intersperse)
+import Data.Maybe (fromJust)
+
+import Network.Mattermost.Types ( TeamId )
 
 import Matterhorn.Draw.Util (renderKeybindingHelp)
 import Matterhorn.Types
-import Matterhorn.Types.KeyEvents
 import Matterhorn.Themes
 
-drawNotifyPrefs :: ChatState -> Widget Name
-drawNotifyPrefs st =
-    let Just form = st^.csCurrentTeam.tsNotifyPrefs
+drawNotifyPrefs :: ChatState -> TeamId -> Widget Name
+drawNotifyPrefs st tId =
+    let form = fromJust $ st^.csTeam(tId).tsNotifyPrefs
         label = forceAttr clientEmphAttr $ str "Notification Preferences"
         formKeys = withDefAttr clientEmphAttr <$> txt <$> ["Tab", "BackTab"]
         bindings = vBox $ hCenter <$> [ renderKeybindingHelp st "Save" [FormSubmitEvent] <+> txt "  " <+>
