@@ -17,18 +17,17 @@ import           Network.Mattermost.Types ( TeamId )
 import           Matterhorn.Draw.Util ( renderKeybindingHelp )
 import           Matterhorn.Types
 import           Matterhorn.Themes
-import           Matterhorn.Types.KeyEvents
 
 -- | Render a tabbed window.
 drawTabbedWindow :: (Eq a, Show a)
-                 => TabbedWindow a
+                 => TabbedWindow ChatState m Name a
                  -> ChatState
+                 -> TeamId
                  -> Widget Name
-drawTabbedWindow w cs =
+drawTabbedWindow w cs tId =
     let cur = getCurrentTabbedWindowEntry w
         tabBody = tweRender cur (twValue w) cs
         title = forceAttr clientEmphAttr $ twtTitle (twTemplate w) (tweValue cur)
-        tId = cs^.csCurrentTeamId
     in centerLayer $
        vLimit (twWindowHeight w) $
        hLimit (twWindowWidth w) $
@@ -47,7 +46,7 @@ keybindingHelp st =
 -- | The scrollable tab bar to show at the top of a tabbed window.
 tabBar :: (Eq a, Show a)
        => TeamId
-       -> TabbedWindow a
+       -> TabbedWindow s m Name a
        -> Widget Name
 tabBar tId w =
     let cur = getCurrentTabbedWindowEntry w
@@ -57,7 +56,7 @@ tabBar tId w =
                           then withDefAttr tabSelectedAttr
                           else withDefAttr tabUnselectedAttr
                 isCurrent = tweValue e == tweValue cur
-                makeVisible = if isCurrent then visible else id
+                makeTabVisible = if isCurrent then visible else id
                 decorateTab v = Widget Fixed Fixed $ do
                     result <- render v
                     let width = Vty.imageWidth (result^.imageL)
@@ -68,7 +67,7 @@ tabBar tId w =
                            render $ vBox [ resultToWidget result
                                          , hLimit width hBorder
                                          ]
-            in makeVisible $
+            in makeTabVisible $
                decorateTab $
                useAttr $
                padLeftRight 2 $
