@@ -1,7 +1,7 @@
 {-# LANGUAGE LambdaCase #-}
+{-# LANGUAGE RankNTypes #-}
 module Matterhorn.State.Attachments
   ( showAttachmentList
-  , resetAttachmentList
   , showAttachmentFileBrowser
   , attachFileByPath
   , tryAddAttachment
@@ -12,7 +12,6 @@ where
 import           Prelude ()
 import           Matterhorn.Prelude
 
-import           Brick ( vScrollToBeginning, viewportScroll )
 import qualified Brick.Widgets.List as L
 import qualified Brick.Widgets.FileBrowser as FB
 import qualified Control.Exception as E
@@ -22,7 +21,7 @@ import           Data.Either ( isRight )
 import           Data.Text ( unpack )
 import qualified Data.Vector as Vector
 import           GHC.Exception ( toException )
-import           Lens.Micro.Platform ( (.=), (%=) )
+import           Lens.Micro.Platform ( (.=), (%=), Lens' )
 import           System.Directory ( doesDirectoryExist, doesFileExist, getDirectoryContents )
 
 import           Matterhorn.Types
@@ -40,43 +39,36 @@ validateAttachmentPath path = bool Nothing (Just path) <$> do
 defaultAttachmentsPath :: Config -> IO (Maybe FilePath)
 defaultAttachmentsPath = maybe (return Nothing) validateAttachmentPath . configDefaultAttachmentPath
 
-showAttachmentList :: MH ()
-showAttachmentList = do
-    lst <- use (csCurrentTeam.tsEditState.cedAttachmentList)
+showAttachmentList :: Lens' ChatState (MessageInterface Name i) -> MH ()
+showAttachmentList which = do
+    lst <- use (which.miEditor.esAttachmentList)
     case length (L.listElements lst) of
-        0 -> showAttachmentFileBrowser
-        _ -> setMode ManageAttachments
+        0 -> showAttachmentFileBrowser which
+        _ -> which.miMode .= ManageAttachments
 
-resetAttachmentList :: MH ()
-resetAttachmentList = do
-    tId <- use csCurrentTeamId
-    let listName = AttachmentList tId
-    csCurrentTeam.tsEditState.cedAttachmentList .= L.list listName mempty 1
-    mh $ vScrollToBeginning $ viewportScroll listName
-
-showAttachmentFileBrowser :: MH ()
-showAttachmentFileBrowser = do
+showAttachmentFileBrowser :: Lens' ChatState (MessageInterface Name i) -> MH ()
+showAttachmentFileBrowser which = do
+    cId <- use (which.miEditor.esChannelId)
     config <- use (csResources.crConfiguration)
-    tId <- use csCurrentTeamId
     filePath <- liftIO $ defaultAttachmentsPath config
-    browser <- liftIO $ Just <$> FB.newFileBrowser FB.selectNonDirectories (AttachmentFileBrowser tId) filePath
-    csCurrentTeam.tsEditState.cedFileBrowser .= browser
-    setMode ManageAttachmentsBrowseFiles
+    browser <- liftIO $ Just <$> FB.newFileBrowser FB.selectNonDirectories (AttachmentFileBrowser cId) filePath
+    which.miEditor.esFileBrowser .= browser
+    which.miMode .= BrowseFiles
 
-attachFileByPath :: Text -> MH ()
-attachFileByPath txtPath = do
+attachFileByPath :: Lens' ChatState (MessageInterface Name i) -> Text -> MH ()
+attachFileByPath which txtPath = do
     let strPath = unpack txtPath
     fileInfo <- liftIO $ FB.getFileInfo strPath strPath
     case FB.fileInfoFileStatus fileInfo of
         Left e -> do
             mhError $ AttachmentException (toException e)
-        Right _ -> tryAddAttachment [fileInfo]
+        Right _ -> tryAddAttachment which [fileInfo]
 
 checkPathIsFile :: FB.FileInfo -> MH Bool
 checkPathIsFile = liftIO . doesFileExist . FB.fileInfoFilePath
 
-tryAddAttachment :: [FB.FileInfo] -> MH ()
-tryAddAttachment entries = do
+tryAddAttachment :: Lens' ChatState (MessageInterface Name i) -> [FB.FileInfo] -> MH ()
+tryAddAttachment which entries = do
     forM_ entries $ \entry -> do
         isFile <- checkPathIsFile entry
         if not isFile
@@ -84,7 +76,7 @@ tryAddAttachment entries = do
             "Error attaching file. It either doesn't exist or is a directory, which is not supported.")
         else do
             -- Is the entry already present? If so, ignore the selection.
-            es <- use (csCurrentTeam.tsEditState.cedAttachmentList.L.listElementsL)
+            es <- use (which.miEditor.esAttachmentList.L.listElementsL)
             let matches = (== FB.fileInfoFilePath entry) .
                               FB.fileInfoFilePath .
                               attachmentDataFileInfo
@@ -93,14 +85,15 @@ tryAddAttachment entries = do
                 Nothing -> do
                     tryReadAttachment entry >>= \case
                         Right a -> do
-                            oldIdx <- use (csCurrentTeam.tsEditState.cedAttachmentList.L.listSelectedL)
+                            oldIdx <- use (which.miEditor.esAttachmentList.L.listSelectedL)
                             let newIdx = if Vector.null es
                                          then Just 0
                                          else oldIdx
-                            csCurrentTeam.tsEditState.cedAttachmentList %= L.listReplace (Vector.snoc es a) newIdx
+                            which.miEditor.esAttachmentList %= L.listReplace (Vector.snoc es a) newIdx
                         Left e -> mhError $ AttachmentException e
 
-    when (not $ null entries) $ setMode Main
+    when (not $ null entries) $
+        which.miMode .= Compose
 
 tryReadAttachment :: FB.FileInfo -> MH (Either E.SomeException AttachmentData)
 tryReadAttachment fi = do
