@@ -1,3 +1,4 @@
+{-# LANGUAGE RankNTypes #-}
 module Matterhorn.State.Autocomplete
   ( AutocompleteContext(..)
   , checkForAutocompletion
@@ -7,6 +8,7 @@ where
 import           Prelude ()
 import           Matterhorn.Prelude
 
+import           Brick ( getName )
 import           Brick.Main ( viewportScroll, vScrollToBeginning )
 import           Brick.Widgets.Edit ( editContentsL )
 import qualified Brick.Widgets.List as L
@@ -15,11 +17,12 @@ import qualified Data.Foldable as F
 import qualified Data.HashMap.Strict as HM
 import           Data.List ( sortBy, partition )
 import qualified Data.Map as M
+import           Data.Maybe ( fromJust )
 import qualified Data.Sequence as Seq
 import qualified Data.Text as T
 import qualified Data.Text.Zipper as Z
 import qualified Data.Vector as V
-import           Lens.Micro.Platform ( (%=), (.=), (.~), _Just, preuse )
+import           Lens.Micro.Platform ( (%=), (.=), (.~), _Just, Traversal' )
 import qualified Skylighting.Types as Sky
 
 import           Network.Mattermost.Types (userId, channelId, Command(..), TeamId)
@@ -55,13 +58,16 @@ data AutocompleteContext =
 -- should cause an autocompletion UI to appear. If so, initiate a server
 -- query or local cache lookup to present the completion alternatives
 -- for the word at the cursor.
-checkForAutocompletion :: AutocompleteContext -> MH ()
-checkForAutocompletion ctx = do
-    result <- getCompleterForInput ctx
+checkForAutocompletion :: Traversal' ChatState (EditState Name)
+                       -> AutocompleteContext
+                       -> MH ()
+checkForAutocompletion which ctx = do
+    result <- getCompleterForInput which ctx
     case result of
-        Nothing -> resetAutocomplete
+        Nothing -> resetAutocomplete which
         Just (ty, runUpdater, searchString) -> do
-            prevResult <- use (csCurrentTeam.tsEditState.cedAutocomplete)
+            prevResult <- join <$> preuse (which.esAutocomplete)
+
             -- We should update the completion state if EITHER:
             --
             -- 1) The type changed
@@ -74,54 +80,68 @@ checkForAutocompletion ctx = do
                                 (maybe True ((== ty) . _acType) prevResult)) ||
                                (maybe False ((/= ty) . _acType) prevResult)
             when shouldUpdate $ do
-                csCurrentTeam.tsEditState.cedAutocompletePending .= Just searchString
+                which.esAutocompletePending .= Just searchString
                 runUpdater ty ctx searchString
 
-getCompleterForInput :: AutocompleteContext
+getCompleterForInput :: Traversal' ChatState (EditState Name)
+                     -> AutocompleteContext
                      -> MH (Maybe (AutocompletionType, AutocompletionType -> AutocompleteContext -> Text -> MH (), Text))
-getCompleterForInput ctx = do
-    z <- use (csCurrentTeam.tsEditState.cedEditor.editContentsL)
+getCompleterForInput which ctx = do
+    maybeZipper <- preuse (which.esEditor.editContentsL)
+    mmTid <- preuse (which.esTeamId)
+    tId <- do
+        case mmTid of
+            Just (Just i) -> return i
+            _ -> fromJust <$> use csCurrentTeamId
+    case maybeZipper of
+        Just z -> do
+            let col = snd $ Z.cursorPosition z
+                curLine = Z.currentLine z
 
-    let col = snd $ Z.cursorPosition z
-        curLine = Z.currentLine z
-
-    return $ case wordAtColumn col curLine of
-        Just (startCol, w)
-            | userSigil `T.isPrefixOf` w ->
-                Just (ACUsers, doUserAutoCompletion, T.tail w)
-            | normalChannelSigil `T.isPrefixOf` w ->
-                Just (ACChannels, doChannelAutoCompletion, T.tail w)
-            | ":" `T.isPrefixOf` w && autocompleteManual ctx ->
-                Just (ACEmoji, doEmojiAutoCompletion, T.tail w)
-            | "```" `T.isPrefixOf` w ->
-                Just (ACCodeBlockLanguage, doSyntaxAutoCompletion, T.drop 3 w)
-            | "/" `T.isPrefixOf` w && startCol == 0 ->
-                Just (ACCommands, doCommandAutoCompletion, T.tail w)
-        _ -> Nothing
+            return $ case wordAtColumn col curLine of
+                Just (startCol, w)
+                    | userSigil `T.isPrefixOf` w ->
+                        Just (ACUsers, doUserAutoCompletion which tId, T.tail w)
+                    | normalChannelSigil `T.isPrefixOf` w ->
+                        Just (ACChannels, doChannelAutoCompletion tId which, T.tail w)
+                    | ":" `T.isPrefixOf` w && autocompleteManual ctx ->
+                        Just (ACEmoji, doEmojiAutoCompletion which, T.tail w)
+                    | "```" `T.isPrefixOf` w ->
+                        Just (ACCodeBlockLanguage, doSyntaxAutoCompletion which, T.drop 3 w)
+                    | "/" `T.isPrefixOf` w && startCol == 0 ->
+                        Just (ACCommands, doCommandAutoCompletion which tId, T.tail w)
+                _ -> Nothing
+        _ -> return Nothing
 
 -- Completion implementations
 
-doEmojiAutoCompletion :: AutocompletionType -> AutocompleteContext -> Text -> MH ()
-doEmojiAutoCompletion ty ctx searchString = do
+doEmojiAutoCompletion :: Traversal' ChatState (EditState Name)
+                      -> AutocompletionType
+                      -> AutocompleteContext
+                      -> Text
+                      -> MH ()
+doEmojiAutoCompletion which ty ctx searchString = do
     session <- getSession
     em <- use (csResources.crEmoji)
-    tId <- use csCurrentTeamId
-    withCachedAutocompleteResults tId ctx ty searchString $
+    withCachedAutocompleteResults which ctx ty searchString $
         doAsyncWith Preempt $ do
             results <- getMatchingEmoji session em searchString
             let alts = EmojiCompletion <$> results
-            return $ Just $ setCompletionAlternatives tId ctx searchString alts ty
+            return $ Just $ setCompletionAlternatives which ctx searchString alts ty
 
-doSyntaxAutoCompletion :: AutocompletionType -> AutocompleteContext -> Text -> MH ()
-doSyntaxAutoCompletion ty ctx searchString = do
+doSyntaxAutoCompletion :: Traversal' ChatState (EditState Name)
+                       -> AutocompletionType
+                       -> AutocompleteContext
+                       -> Text
+                       -> MH ()
+doSyntaxAutoCompletion which ty ctx searchString = do
     mapping <- use (csResources.crSyntaxMap)
-    tId <- use csCurrentTeamId
     let allNames = Sky.sShortname <$> M.elems mapping
         (prefixed, notPrefixed) = partition isPrefixed $ filter match allNames
         match = (((T.toLower searchString) `T.isInfixOf`) . T.toLower)
         isPrefixed = (((T.toLower searchString) `T.isPrefixOf`) . T.toLower)
         alts = SyntaxCompletion <$> (sort prefixed <> sort notPrefixed)
-    setCompletionAlternatives tId ctx searchString alts ty
+    setCompletionAlternatives which ctx searchString alts ty
 
 -- | This list of server commands should be hidden because they make
 -- assumptions about a web-based client or otherwise just don't make
@@ -170,13 +190,17 @@ hiddenCommand c = (T.toLower $ commandTrigger c) `elem` hiddenServerCommands
 isDeletedCommand :: Command -> Bool
 isDeletedCommand cmd = commandDeleteAt cmd > commandCreateAt cmd
 
-doCommandAutoCompletion :: AutocompletionType -> AutocompleteContext -> Text -> MH ()
-doCommandAutoCompletion ty ctx searchString = do
+doCommandAutoCompletion :: Traversal' ChatState (EditState Name)
+                        -> TeamId
+                        -> AutocompletionType
+                        -> AutocompleteContext
+                        -> Text
+                        -> MH ()
+doCommandAutoCompletion which tId ty ctx searchString = do
     session <- getSession
-    myTid <- use csCurrentTeamId
 
-    mCache <- preuse (csTeam(myTid).tsEditState.cedAutocomplete._Just.acCachedResponses)
-    mActiveTy <- preuse (csTeam(myTid).tsEditState.cedAutocomplete._Just.acType)
+    mCache <- preuse (which.esAutocomplete._Just.acCachedResponses)
+    mActiveTy <- preuse (which.esAutocomplete._Just.acType)
 
     -- Command completion works a little differently than the other
     -- modes. To do command autocompletion, we want to query the server
@@ -212,7 +236,8 @@ doCommandAutoCompletion ty ctx searchString = do
                     mkAlt (Cmd name desc args _) =
                         (Client, name, printArgSpec args, desc)
 
-                serverCommands <- MM.mmListCommandsForTeam myTid False session
+                serverCommands <- MM.mmListCommandsForTeam tId False session
+
                 let filteredServerCommands =
                         filter (\c -> not (hiddenCommand c || isDeletedCommand c)) $
                         F.toList serverCommands
@@ -230,19 +255,19 @@ doCommandAutoCompletion ty ctx searchString = do
 
                 return $ Just $ do
                     -- Store the complete list of alterantives in the cache
-                    setCompletionAlternatives myTid ctx serverResponseKey alts ty
+                    setCompletionAlternatives which ctx serverResponseKey alts ty
 
                     -- Also store the list of alternatives specific to
                     -- this search string
                     let newAlts = sortBy (compareCommandAlts searchString) $
                                   filter matches alts
-                    setCompletionAlternatives myTid ctx searchString newAlts ty
+                    setCompletionAlternatives which ctx searchString newAlts ty
 
        else case entry of
            Just alts | mActiveTy == Just ACCommands ->
                let newAlts = sortBy (compareCommandAlts searchString) $
                              filter matches alts
-               in setCompletionAlternatives myTid ctx searchString newAlts ty
+               in setCompletionAlternatives which ctx searchString newAlts ty
            _ -> return ()
 
 compareCommandAlts :: Text -> AutocompleteAlternative -> AutocompleteAlternative -> Ordering
@@ -257,14 +282,17 @@ compareCommandAlts s (CommandCompletion _ nameA _ _)
             else GT
 compareCommandAlts _ _ _ = LT
 
-doUserAutoCompletion :: AutocompletionType -> AutocompleteContext -> Text -> MH ()
-doUserAutoCompletion ty ctx searchString = do
+doUserAutoCompletion :: Traversal' ChatState (EditState Name)
+                     -> TeamId
+                     -> AutocompletionType
+                     -> AutocompleteContext
+                     -> Text
+                     -> MH ()
+doUserAutoCompletion which tId ty ctx searchString = do
     session <- getSession
-    tId <- use csCurrentTeamId
     myUid <- gets myUserId
-    cId <- use (csCurrentChannelId(tId))
-
-    withCachedAutocompleteResults tId ctx ty searchString $
+    cId <- fromJust <$> preuse (which.esChannelId)
+    withCachedAutocompleteResults which ctx ty searchString $
         doAsyncWith Preempt $ do
             ac <- MM.mmAutocompleteUsers (Just tId) (Just cId) searchString session
 
@@ -281,22 +309,26 @@ doUserAutoCompletion ty ctx searchString = do
                          , (T.toLower searchString) `T.isPrefixOf` specialMentionName m
                          ]
 
-            return $ Just $ setCompletionAlternatives tId ctx searchString (alts <> extras) ty
+            return $ Just $ setCompletionAlternatives which ctx searchString (alts <> extras) ty
 
-doChannelAutoCompletion :: AutocompletionType -> AutocompleteContext -> Text -> MH ()
-doChannelAutoCompletion ty ctx searchString = do
+doChannelAutoCompletion :: TeamId
+                        -> Traversal' ChatState (EditState Name)
+                        -> AutocompletionType
+                        -> AutocompleteContext
+                        -> Text
+                        -> MH ()
+doChannelAutoCompletion tId which ty ctx searchString = do
     session <- getSession
-    tId <- use csCurrentTeamId
     cs <- use csChannels
 
-    withCachedAutocompleteResults tId ctx ty searchString $ do
+    withCachedAutocompleteResults which ctx ty searchString $ do
         doAsyncWith Preempt $ do
             results <- MM.mmAutocompleteChannels tId searchString session
             let alts = F.toList $ (ChannelCompletion True <$> inChannels) <>
                                   (ChannelCompletion False <$> notInChannels)
                 (inChannels, notInChannels) = Seq.partition isMember results
                 isMember c = isJust $ findChannelById (channelId c) cs
-            return $ Just $ setCompletionAlternatives tId ctx searchString alts ty
+            return $ Just $ setCompletionAlternatives which ctx searchString alts ty
 
 -- Utility functions
 
@@ -304,7 +336,7 @@ doChannelAutoCompletion ty ctx searchString = do
 -- a given search string. If the cache contains no such entry (keyed
 -- on search string), run the specified action, which is assumed to be
 -- responsible for fetching the completion results from the server.
-withCachedAutocompleteResults :: TeamId
+withCachedAutocompleteResults :: Traversal' ChatState (EditState Name)
                               -> AutocompleteContext
                               -- ^ The autocomplete context
                               -> AutocompletionType
@@ -316,61 +348,66 @@ withCachedAutocompleteResults :: TeamId
                               -> MH ()
                               -- ^ The action to execute on a cache miss
                               -> MH ()
-withCachedAutocompleteResults tId ctx ty searchString act = do
-    mCache <- preuse (csTeam(tId).tsEditState.cedAutocomplete._Just.acCachedResponses)
-    mActiveTy <- preuse (csTeam(tId).tsEditState.cedAutocomplete._Just.acType)
+withCachedAutocompleteResults which ctx ty searchString act = do
+    mCache <- preuse (which.esAutocomplete._Just.acCachedResponses)
+    mActiveTy <- preuse (which.esAutocomplete._Just.acType)
 
     case Just ty == mActiveTy of
         True ->
             -- Does the cache have results for this search string? If
             -- so, use them; otherwise invoke the specified action.
             case HM.lookup searchString =<< mCache of
-                Just alts -> setCompletionAlternatives tId ctx searchString alts ty
+                Just alts -> setCompletionAlternatives which ctx searchString alts ty
                 Nothing -> act
         False -> act
 
-setCompletionAlternatives :: TeamId
+setCompletionAlternatives :: Traversal' ChatState (EditState Name)
                           -> AutocompleteContext
                           -> Text
                           -> [AutocompleteAlternative]
                           -> AutocompletionType
                           -> MH ()
-setCompletionAlternatives tId ctx searchString alts ty = do
-    let list = L.list (CompletionList tId) (V.fromList $ F.toList alts) 1
-        state = AutocompleteState { _acPreviousSearchString = searchString
-                                  , _acCompletionList =
-                                      list & L.listSelectedL .~ Nothing
-                                  , _acCachedResponses = HM.fromList [(searchString, alts)]
-                                  , _acType = ty
-                                  }
+setCompletionAlternatives which ctx searchString alts ty = do
+    mVal <- preuse which
+    case mVal of
+        Nothing -> return ()
+        Just esVal -> do
+            let list = L.list (CompletionList $ getName $ esVal^.esEditor) (V.fromList $ F.toList alts) 1
+                pending = esVal^.esAutocompletePending
+                state = AutocompleteState { _acPreviousSearchString = searchString
+                                          , _acCompletionList =
+                                              list & L.listSelectedL .~ Nothing
+                                          , _acCachedResponses = HM.fromList [(searchString, alts)]
+                                          , _acType = ty
+                                          }
 
-    pending <- use (csTeam(tId).tsEditState.cedAutocompletePending)
-    case pending of
-        Just val | val == searchString -> do
+            case pending of
+                Just val | val == searchString -> do
 
-            -- If there is already state, update it, but also cache the
-            -- search results.
-            csTeam(tId).tsEditState.cedAutocomplete %= \prev ->
-                let newState = case prev of
-                        Nothing ->
-                            state
-                        Just oldState ->
-                            state & acCachedResponses .~
-                                HM.insert searchString alts (oldState^.acCachedResponses)
-                in Just newState
+                    -- If there is already state, update it, but also cache the
+                    -- search results.
+                    which.esAutocomplete %= \prev ->
+                        let newState = case prev of
+                                Nothing ->
+                                    state
+                                Just oldState ->
+                                    state & acCachedResponses .~
+                                        HM.insert searchString alts (oldState^.acCachedResponses)
+                        in Just newState
 
-            mh $ vScrollToBeginning $ viewportScroll $ CompletionList tId
+                    mh $ vScrollToBeginning $ viewportScroll $ CompletionList $ getName $ esVal^.esEditor
 
-            when (autocompleteFirstMatch ctx) $
-                tabComplete Forwards
-        _ ->
-            -- Do not update the state if this result does not
-            -- correspond to the search string we used most recently.
-            -- This happens when the editor changes faster than the
-            -- async completion responses arrive from the server. If we
-            -- don't check this, we show completion results that are
-            -- wrong for the editor state.
-            return ()
+                    when (autocompleteFirstMatch ctx) $
+                        tabComplete which Forwards
+                _ ->
+                    -- Do not update the state if this result does not
+                    -- correspond to the search string we used most
+                    -- recently. This happens when the editor changes
+                    -- faster than the async completion responses arrive
+                    -- from the server. If we don't check this, we show
+                    -- completion results that are wrong for the editor
+                    -- state.
+                    return ()
 
 wordAtColumn :: Int -> Text -> Maybe (Int, Text)
 wordAtColumn i t =
