@@ -13,6 +13,7 @@ import           Brick.Main ( getVtyHandle, invalidateCache, invalidateCacheEntr
 import qualified Data.HashMap.Strict as HM
 import qualified Data.Sequence as Seq
 import           Data.Time.Clock ( getCurrentTime )
+import           Data.Maybe ( fromJust )
 import qualified Graphics.Vty as Vty
 import           Lens.Micro.Platform
 
@@ -44,12 +45,18 @@ updateSidebar mTid = do
     -- Schedule the current team's sidebar for user status updates at
     -- the end of this MH action. This is okay because all team sidebars
     -- should include the same set of DM channels.
-    z <- use (csCurrentTeam.tsFocus)
-    myId <- gets myUserId
-    scheduleUserStatusFetches $ myId : userIdsFromZipper z
+    withCurrentTeam $ \tId -> do
+        z <- use (csTeam(tId).tsFocus)
+        myId <- gets myUserId
+        scheduleUserStatusFetches $ myId : userIdsFromZipper z
 
     updateWindowTitle
 
+    -- Also refresh the team zipper since a sidebar update due to a new
+    -- message arrival could cause the team ordering to change if the
+    -- team ordering is set to show unread teams first.
+    refreshTeamZipper
+
 updateWindowTitle :: MH ()
 updateWindowTitle = do
     -- Update the window title based on the unread status of the
@@ -78,12 +85,14 @@ updateTeamSidebar tId = do
 
     -- Update the zipper
     cs <- use csChannels
+    sorting <- use (csTeam(tId).tsChannelListSorting)
     us <- getUsers
     prefs <- use (csResources.crUserPreferences)
+    hidden <- use csHiddenChannelGroups
     now <- liftIO getCurrentTime
     config <- use (csResources.crConfiguration)
 
-    let zl = mkChannelZipperList now config tId cconfig prefs cs us
+    let zl = mkChannelZipperList sorting now config tId cconfig prefs hidden cs us
         compareEntries mOld new = (channelListEntryChannelId <$> mOld) == Just (channelListEntryChannelId new)
     csTeam(tId).tsFocus %= Z.updateListBy compareEntries zl
 
@@ -93,7 +102,7 @@ updateTeamSidebar tId = do
     -- that it gets loaded.
     newCid <- use (csCurrentChannelId tId)
     when (newCid /= oldCid) $
-        fetchVisibleIfNeeded
+        fetchVisibleIfNeeded tId
 
 toggleChannelListVisibility :: MH ()
 toggleChannelListVisibility = do
@@ -127,24 +136,20 @@ showChannelInSidebar cId setPending = do
             -- displayable, and the UI should always prefer to SATISFY
             -- the user's latest request over any pending/background
             -- task.
-            csCurrentTeam.tsPendingChannelChange .= Nothing
-
             now <- liftIO getCurrentTime
             csChannel(cId).ccInfo.cdSidebarShowOverride .= Just now
             updateSidebar (ch^.ccInfo.cdTeamId)
 
-            curTid <- use csCurrentTeamId
-            let tId = fromMaybe curTid (ch^.ccInfo.cdTeamId)
-
             case ch^.ccInfo.cdType of
                 Direct -> do
-                    let Just uId = ch^.ccInfo.cdDMUserId
+                    let uId = fromJust $ ch^.ccInfo.cdDMUserId
                     case dmChannelShowPreference prefs uId of
                         Just False -> do
                             let pref = showDirectChannelPref (me^.userIdL) uId True
                             when setPending $
-                                csCurrentTeam.tsPendingChannelChange .=
-                                    Just (ChangeByChannelId tId (ch^.ccInfo.cdChannelId) Nothing)
+                                withCurrentTeam $ \tId ->
+                                    csTeam(tId).tsPendingChannelChange .=
+                                        Just (ChangeByChannelId tId (ch^.ccInfo.cdChannelId) Nothing)
                             doAsyncWith Preempt $ do
                                 MM.mmSaveUsersPreferences UserMe (Seq.singleton pref) session
                                 return Nothing
@@ -155,8 +160,9 @@ showChannelInSidebar cId setPending = do
                         Just False -> do
                             let pref = showGroupChannelPref cId (me^.userIdL)
                             when setPending $
-                                csCurrentTeam.tsPendingChannelChange .=
-                                    Just (ChangeByChannelId tId (ch^.ccInfo.cdChannelId) Nothing)
+                                withCurrentTeam $ \tId ->
+                                    csTeam(tId).tsPendingChannelChange .=
+                                        Just (ChangeByChannelId tId (ch^.ccInfo.cdChannelId) Nothing)
                             doAsyncWith Preempt $ do
                                 MM.mmSaveUsersPreferences UserMe (Seq.singleton pref) session
                                 return Nothing
