@@ -17,11 +17,18 @@ module Matterhorn.State.Common
   , postErrorMessage'
   , addEmoteFormatting
   , removeEmoteFormatting
+  , toggleMouseMode
 
   , fetchMentionedUsers
   , doPendingUserFetches
   , doPendingUserStatusFetches
 
+  , setThreadOrientationByName
+
+  -- Cache management
+  , invalidateChannelRenderingCache
+  , invalidateMessageRenderingCacheByPostId
+
   , module Matterhorn.State.Async
   )
 where
@@ -29,21 +36,23 @@ where
 import           Prelude ()
 import           Matterhorn.Prelude
 
-import           Brick.Main ( invalidateCacheEntry )
+import           Brick.Main ( invalidateCacheEntry, invalidateCache, getVtyHandle )
 import           Control.Concurrent ( MVar, putMVar, forkIO )
 import qualified Control.Concurrent.STM as STM
 import           Control.Exception ( SomeException, try )
 import qualified Data.ByteString as BS
+import qualified Data.ByteString.Lazy as BSL
 import qualified Data.HashMap.Strict as HM
 import qualified Data.Sequence as Seq
 import qualified Data.Set as Set
 import qualified Data.Text as T
+import qualified Graphics.Vty as Vty
 import           Lens.Micro.Platform ( (.=), (%=), (%~), (.~) )
 import           System.Directory ( createDirectoryIfMissing )
 import           System.Environment.XDG.BaseDir ( getUserCacheDir )
 import           System.Exit ( ExitCode(..) )
 import           System.FilePath
-import           System.IO ( hGetContents, hFlush, hPutStrLn )
+import           System.IO ( hGetContents, hFlush )
 import           System.Process ( proc, std_in, std_out, std_err, StdStream(..)
                                 , createProcess, waitForProcess )
 
@@ -78,10 +87,12 @@ installMessagesFromPosts mTId postCollection = do
       Nothing -> return Nothing
       Just tId -> Just <$> getServerBaseUrl tId
 
+  hostname <- use (csResources.crConn.cdHostnameL)
+
   -- Build the ordered list of posts. Note that postsOrder lists the
   -- posts most recent first, but we want most recent last.
   let postsInOrder = findPost <$> (Seq.reverse $ postsOrder postCollection)
-      mkClientPost p = toClientPost mBaseUrl p (postId <$> parent p)
+      mkClientPost p = toClientPost hostname mBaseUrl p (postId <$> parent p)
       clientPosts = mkClientPost <$> postsInOrder
 
       addNext cp (msgs, us) =
@@ -116,9 +127,11 @@ updatePostMap mTId postCollection = do
       Nothing -> return Nothing
       Just tId -> Just <$> getServerBaseUrl tId
 
+  hostname <- use (csResources.crConn.cdHostnameL)
+
   let postMap = HM.fromList
           [ ( pId
-            , fst $ clientPostToMessage (toClientPost mBaseUrl x Nothing)
+            , fst $ clientPostToMessage (toClientPost hostname mBaseUrl x Nothing)
             )
           | (pId, x) <- HM.toList (postCollection^.postsPostsL)
           ]
@@ -127,21 +140,21 @@ updatePostMap mTId postCollection = do
 -- | Add a 'ClientMessage' to the current channel's message list
 addClientMessage :: ClientMessage -> MH ()
 addClientMessage msg = do
-  tId <- use csCurrentTeamId
-  cid <- use (csCurrentChannelId(tId))
-  uuid <- generateUUID
-  let addCMsg = ccContents.cdMessages %~
-          (addMessage $ clientMessageToMessage msg & mMessageId .~ Just (MessageUUID uuid))
-  csChannels %= modifyChannelById cid addCMsg
+    withCurrentTeam $ \tId -> do
+        withCurrentChannel tId $ \cid _ -> do
+            uuid <- generateUUID
+            let addCMsg = ccMessageInterface.miMessages %~
+                    (addMessage $ clientMessageToMessage msg & mMessageId .~ Just (MessageUUID uuid))
+            csChannels %= modifyChannelById cid addCMsg
 
-  mh $ invalidateCacheEntry $ ChannelMessages cid
-  mh $ invalidateCacheEntry $ ChannelSidebar tId
+            invalidateChannelRenderingCache cid
+            mh $ invalidateCacheEntry $ ChannelSidebar tId
 
-  let msgTy = case msg^.cmType of
-        Error -> LogError
-        _     -> LogGeneral
+            let msgTy = case msg^.cmType of
+                  Error -> LogError
+                  _     -> LogGeneral
 
-  mhLog msgTy $ T.pack $ show msg
+            mhLog msgTy $ T.pack $ show msg
 
 -- | Add a new 'ClientMessage' representing an error message to
 --   the current channel's message list
@@ -157,12 +170,17 @@ postErrorMessage' err =
 
 postErrorMessageIO :: Text -> ChatState -> IO ChatState
 postErrorMessageIO err st = do
-  msg <- newClientMessage Error err
-  uuid <- generateUUID_IO
-  let cId = st ^. csCurrentChannelId (st^.csCurrentTeamId)
-      addEMsg = ccContents.cdMessages %~
-          (addMessage $ clientMessageToMessage msg & mMessageId .~ Just (MessageUUID uuid))
-  return $ st & csChannels %~ modifyChannelById cId addEMsg
+  case st^.csCurrentTeamId of
+      Nothing -> return st
+      Just tId -> do
+          case st^.csCurrentChannelId(tId) of
+              Nothing -> return st
+              Just cId -> do
+                  msg <- newClientMessage Error err
+                  uuid <- generateUUID_IO
+                  let addEMsg = ccMessageInterface.miMessages %~
+                          (addMessage $ clientMessageToMessage msg & mMessageId .~ Just (MessageUUID uuid))
+                  return $ st & csChannels %~ modifyChannelById cId addEMsg
 
 openFilePath :: FilePath -> MH ()
 openFilePath path = openWithOpener (return $ Right path)
@@ -175,7 +193,9 @@ openWithOpener getTarget = do
             mhError $ ConfigOptionMissing "urlOpenCommand"
         Just urlOpenCommand -> do
             targetResult <- getTarget
-
+            let cmdWords = T.words urlOpenCommand
+                (cmds, args) = splitAt 1 (T.unpack <$> cmdWords)
+                cmd = if null cmds then "$BROWSER" else head cmds
             case targetResult of
                 Left e -> do
                     mhError e
@@ -189,8 +209,9 @@ openWithOpener getTarget = do
                         False -> do
                             outputChan <- use (csResources.crSubprocessLog)
                             doAsyncWith Preempt $ do
-                                runLoggedCommand outputChan (T.unpack urlOpenCommand)
-                                                 [target] Nothing Nothing
+                                runLoggedCommand outputChan cmd
+                                                 (args <> [target])
+                                                 Nothing Nothing
                                 return Nothing
                         True -> do
                             -- If there isn't a new message cutoff
@@ -199,25 +220,25 @@ openWithOpener getTarget = do
                             -- their interactive URL opener, when they
                             -- return, any messages that arrive in the
                             -- current channel will be displayed as new.
-                            curChan <- use csCurrentChannel
-                            let msgs = curChan^.ccContents.cdMessages
-                            case findLatestUserMessage isEditable msgs of
-                                Nothing -> return ()
-                                Just m ->
-                                    case m^.mOriginalPost of
+                            withCurrentTeam $ \tId -> do
+                                withCurrentChannel tId $ \cId curChan -> do
+                                    let msgs = curChan^.ccMessageInterface.miMessages
+                                    case findLatestUserMessage isEditable msgs of
                                         Nothing -> return ()
-                                        Just p ->
-                                            case curChan^.ccInfo.cdNewMessageIndicator of
-                                                Hide ->
-                                                    csCurrentChannel.ccInfo.cdNewMessageIndicator .= (NewPostsAfterServerTime (p^.postCreateAtL))
-                                                _ -> return ()
-                            -- No need to add a gap here: the websocket
-                            -- disconnect/reconnect events will
-                            -- automatically handle management of
-                            -- messages delivered while suspended.
+                                        Just m ->
+                                            case m^.mOriginalPost of
+                                                Nothing -> return ()
+                                                Just p ->
+                                                    case curChan^.ccInfo.cdNewMessageIndicator of
+                                                        Hide -> csChannel(cId).ccInfo.cdNewMessageIndicator .= (NewPostsAfterServerTime (p^.postCreateAtL))
+                                                        _ -> return ()
 
+                            -- No need to add a gap here: the websocket
+                            -- disconnect/reconnect events will automatically
+                            -- handle management of messages delivered while
+                            -- suspended.
                             mhSuspendAndResume $ \st -> do
-                                result <- runInteractiveCommand (T.unpack urlOpenCommand) [target]
+                                result <- runInteractiveCommand cmd (args <> [target])
 
                                 let waitForKeypress = do
                                         putStrLn "Press any key to return to Matterhorn."
@@ -234,7 +255,9 @@ openWithOpener getTarget = do
                                                    " exited with non-zero status " <> show code
                                         waitForKeypress
 
-                                return $ setMode' Main st
+                                return $ case st^.csCurrentTeamId of
+                                    Nothing -> st
+                                    Just tId -> pushMode' tId Main st
 
 runInteractiveCommand :: String
                       -> [String]
@@ -257,7 +280,7 @@ runLoggedCommand :: STM.TChan ProgramOutput
                  -- ^ The program name
                  -> [String]
                  -- ^ Arguments
-                 -> Maybe String
+                 -> Maybe BSL.ByteString
                  -- ^ The stdin to send, if any
                  -> Maybe (MVar ProgramOutput)
                  -- ^ Where to put the program output when it is ready
@@ -277,9 +300,11 @@ runLoggedCommand outputChan cmd args mInput mOutputVar = void $ forkIO $ do
         Right (stdinResult, Just outh, Just errh, ph) -> do
             case stdinResult of
                 Just inh -> do
-                    let Just input = mInput
-                    hPutStrLn inh input
-                    hFlush inh
+                    case mInput of
+                        Just input -> do
+                            BSL.hPut inh input
+                            hFlush inh
+                        Nothing -> return ()
                 Nothing -> return ()
 
             ec <- waitForProcess ph
@@ -399,3 +424,41 @@ fetchUsers rawUsernames uids = do
                         forM_ results (\u -> addNewUser $ userInfoFromUser u True)
 
             return $ Just $ act1 >> act2
+
+invalidateChannelRenderingCache :: ChannelId -> MH ()
+invalidateChannelRenderingCache cId = do
+    mh $ invalidateCacheEntry $ MessageInterfaceMessages $ MessageInput cId
+    mh $ invalidateCacheEntry $ MessageInterfaceMessages $ ThreadMessageInput cId
+
+invalidateMessageRenderingCacheByPostId :: PostId -> MH ()
+invalidateMessageRenderingCacheByPostId pId = do
+    mh $ invalidateCacheEntry $ RenderedMessage $ MessagePostId pId
+
+setThreadOrientationByName :: T.Text -> MH ()
+setThreadOrientationByName o = do
+    let o' = T.strip $ T.toLower o
+    new <- case o' of
+        "above" -> return $ Just ThreadAbove
+        "below" -> return $ Just ThreadBelow
+        "left"  -> return $ Just ThreadLeft
+        "right" -> return $ Just ThreadRight
+        _ -> do
+            postErrorMessage' $ T.pack $ "Invalid orientation: " <> show o
+            return Nothing
+
+    case new of
+        Nothing -> return ()
+        Just n -> do
+            csResources.crConfiguration.configThreadOrientationL .= n
+            postInfoMessage $ "Thread orientation set to " <> o'
+            mh invalidateCache
+
+toggleMouseMode :: MH ()
+toggleMouseMode = do
+    vty <- mh getVtyHandle
+    csResources.crConfiguration.configMouseModeL %= not
+    newMode <- use (csResources.crConfiguration.configMouseModeL)
+    liftIO $ Vty.setMode (Vty.outputIface vty) Vty.Mouse newMode
+    postInfoMessage $ if newMode
+                      then "Mouse input is now enabled."
+                      else "Mouse input is now disabled."
