@@ -16,26 +16,27 @@ module Matterhorn.State.Messages
   , toggleMessageTimestamps
   , toggleVerbatimBlockTruncation
   , jumpToPost
+  , addMessageToState
   )
 where
 
 import           Prelude ()
 import           Matterhorn.Prelude
 
-import           Brick.Main ( getVtyHandle, invalidateCacheEntry, invalidateCache )
+import           Brick.Main ( getVtyHandle, invalidateCache )
 import qualified Brick.Widgets.FileBrowser as FB
 import           Control.Exception ( SomeException, try )
 import qualified Data.Aeson as A
-import qualified Data.ByteString.Lazy.Char8 as BL8
+import qualified Data.ByteString.Lazy as BSL
 import qualified Data.Foldable as F
 import qualified Data.HashMap.Strict as HM
 import qualified Data.Set as Set
 import qualified Data.Sequence as Seq
 import qualified Data.Text as T
 import           Graphics.Vty ( outputIface )
-import           Graphics.Vty.Output.Interface ( ringTerminalBell )
-import           Lens.Micro.Platform ( Traversal', (.=), (%=), (%~), (.~), (^?)
-                                     , to, at, traversed, filtered, ix, _1 )
+import           Graphics.Vty.Output ( ringTerminalBell )
+import           Lens.Micro.Platform ( Traversal', (.=), (%=), (%~), (.~)
+                                     , to, at, traversed, filtered, ix, _1, _Just )
 
 import           Network.Mattermost
 import qualified Network.Mattermost.Endpoints as MM
@@ -44,8 +45,10 @@ import           Network.Mattermost.Types
 
 import           Matterhorn.Constants
 import           Matterhorn.State.Channels
+import           Matterhorn.State.ChannelList ( updateSidebar )
 import           Matterhorn.State.Common
-import           Matterhorn.State.Reactions
+import           Matterhorn.State.ThreadWindow
+import           Matterhorn.State.MessageSelect
 import           Matterhorn.State.Users
 import           Matterhorn.TimeUtils
 import           Matterhorn.Types
@@ -69,7 +72,7 @@ addDisconnectGaps :: MH ()
 addDisconnectGaps = mapM_ onEach . filteredChannelIds (const True) =<< use csChannels
     where onEach c = do addEndGap c
                         clearPendingFlags c
-                        mh $ invalidateCacheEntry (ChannelMessages c)
+                        invalidateChannelRenderingCache c
 
 -- | Websocket was disconnected, so all channels may now miss some
 -- messages
@@ -98,11 +101,11 @@ toggleVerbatimBlockTruncation = do
     csVerbatimTruncateSetting %= toggle
 
 clearPendingFlags :: ChannelId -> MH ()
-clearPendingFlags c = csChannel(c).ccContents.cdFetchPending .= False
+clearPendingFlags c = csChannel(c).ccInfo.cdFetchPending .= False
 
 addEndGap :: ChannelId -> MH ()
 addEndGap cId = withChannel cId $ \chan ->
-    let lastmsg_ = chan^.ccContents.cdMessages.to reverseMessages.to lastMsg
+    let lastmsg_ = chan^.ccMessageInterface.miMessages.to reverseMessages.to lastMsg
         lastIsGap = maybe False isGap lastmsg_
         gapMsg = newGapMessage timeJustAfterLast
         timeJustAfterLast = maybe t0 (justAfter . _mDate) lastmsg_
@@ -111,7 +114,7 @@ addEndGap cId = withChannel cId $ \chan ->
                         (T.pack "Disconnected. Will refresh when connected.")
                         (C UnknownGapAfter)
     in unless lastIsGap
-           (csChannels %= modifyChannelById cId (ccContents.cdMessages %~ addMessage gapMsg))
+           (csChannels %= modifyChannelById cId (ccMessageInterface.miMessages %~ addMessage gapMsg))
 
 lastMsg :: RetrogradeMessages -> Maybe Message
 lastMsg = withFirstMessage id
@@ -174,12 +177,17 @@ editMessage new = do
             Nothing -> return Nothing
             Just tId -> Just <$> getServerBaseUrl tId
 
-        let (msg, mentionedUsers) = clientPostToMessage (toClientPost mBaseUrl new (new^.postRootIdL))
+        hostname <- use (csResources.crConn.cdHostnameL)
+
+        let (msg, mentionedUsers) = clientPostToMessage (toClientPost hostname mBaseUrl new (new^.postRootIdL))
             isEditedMessage m = m^.mMessageId == Just (MessagePostId $ new^.postIdL)
 
-        csChannel (new^.postChannelIdL) . ccContents . cdMessages . traversed . filtered isEditedMessage .= msg
-        mh $ invalidateCacheEntry (ChannelMessages $ new^.postChannelIdL)
-        mh $ invalidateCacheEntry $ RenderedMessage $ MessagePostId $ postId new
+        csChannel (new^.postChannelIdL) . ccMessageInterface.miMessages . traversed . filtered isEditedMessage .= msg
+
+        invalidateChannelRenderingCache $ new^.postChannelIdL
+        invalidateMessageRenderingCacheByPostId $ postId new
+
+        editPostInOpenThread mTId new msg
 
         fetchMentionedUsers mentionedUsers
 
@@ -187,8 +195,6 @@ editMessage new = do
             csChannel (new^.postChannelIdL) %= adjustEditedThreshold new
 
         csPostMap.ix(postId new) .= msg
-        asyncFetchReactionsForPost (postChannelId new) new
-        asyncFetchAttachments new
 
 deleteMessage :: Post -> MH ()
 deleteMessage new = do
@@ -196,10 +202,34 @@ deleteMessage new = do
                              isReplyTo (new^.postIdL) m
         chan :: Traversal' ChatState ClientChannel
         chan = csChannel (new^.postChannelIdL)
-    chan.ccContents.cdMessages.traversed.filtered isDeletedMessage %= (& mDeleted .~ True)
+    chan.ccMessageInterface.miMessages.traversed.filtered isDeletedMessage %= (& mDeleted .~ True)
     chan %= adjustUpdated new
-    mh $ invalidateCacheEntry (ChannelMessages $ new^.postChannelIdL)
-    mh $ invalidateCacheEntry $ RenderedMessage $ MessagePostId $ postId new
+
+    withChannel (new^.postChannelIdL) $ \ch -> do
+        case ch^.ccInfo.cdTeamId of
+            Nothing -> return ()
+            Just tId -> deletePostFromOpenThread tId new
+
+    invalidateChannelRenderingCache $ new^.postChannelIdL
+    invalidateMessageRenderingCacheByPostId $ postId new
+
+deletePostFromOpenThread :: TeamId -> Post -> MH ()
+deletePostFromOpenThread tId p = do
+    let isDeletedMessage m = m^.mMessageId == Just (MessagePostId $ p^.postIdL) ||
+                             isReplyTo (p^.postIdL) m
+
+    -- If the post being deleted is in the thread, we just need to
+    -- remove it from the thread view. But if this effectively empties
+    -- the thread, that's because this was the root. In that case we
+    -- need to close down the window.
+    threadInterfaceDeleteWhere tId (p^.postChannelIdL) isDeletedMessage
+
+    ti <- use (csTeam(tId).tsThreadInterface)
+    when (isJust ti) $ do
+        isEmpty <- threadInterfaceEmpty tId
+        when isEmpty $ do
+            closeThreadWindow tId
+            postInfoMessage "The thread you were viewing was deleted."
 
 addNewPostedMessage :: PostToAdd -> MH ()
 addNewPostedMessage p =
@@ -215,7 +245,7 @@ addNewPostedMessage p =
 -- message following the added block of messages.
 addObtainedMessages :: ChannelId -> Int -> Bool -> Posts -> MH PostProcessMessageAdd
 addObtainedMessages cId reqCnt addTrailingGap posts = do
-  mh $ invalidateCacheEntry (ChannelMessages cId)
+  invalidateChannelRenderingCache cId
   if null $ posts^.postsOrderL
   then do when addTrailingGap $
             -- Fetched at the end of the channel, but nothing was
@@ -223,7 +253,7 @@ addObtainedMessages cId reqCnt addTrailingGap posts = do
             -- with no messages in it.  Need to remove any gaps that
             -- exist at the end of the channel.
             csChannels %= modifyChannelById cId
-              (ccContents.cdMessages %~
+              (ccMessageInterface.miMessages %~
                \msgs -> let startPoint = join $ _mMessageId <$> getLatestPostMsg msgs
                         in fst $ removeMatchesFromSubset isGap startPoint Nothing msgs)
           return NoAction
@@ -242,7 +272,7 @@ addObtainedMessages cId reqCnt addTrailingGap posts = do
             earliestDate = postCreateAt $ (posts^.postsPostsL) HM.! earliestPId
             latestDate = postCreateAt $ (posts^.postsPostsL) HM.! latestPId
 
-            localMessages = chan^.ccContents . cdMessages
+            localMessages = chan^.ccMessageInterface.miMessages
 
             -- Get a list of the duplicated message PostIds between
             -- the messages already in the channel and the new posts
@@ -350,7 +380,7 @@ addObtainedMessages cId reqCnt addTrailingGap posts = do
         -- Do this with the updated copy of the channel's messages.
 
         withChannelOrDefault cId () $ \updchan -> do
-          let updMsgs = updchan ^. ccContents . cdMessages
+          let updMsgs = updchan ^. ccMessageInterface.miMessages
 
           -- Remove any gaps in the added region.  If there was an
           -- active message selection and it is one of the removed
@@ -362,12 +392,12 @@ addObtainedMessages cId reqCnt addTrailingGap posts = do
           let (resultMessages, removedMessages) =
                 removeMatchesFromSubset isGap removeStart removeEnd updMsgs
           csChannels %= modifyChannelById cId
-            (ccContents.cdMessages .~ resultMessages)
+            (ccMessageInterface.miMessages .~ resultMessages)
 
           let processTeam tId = do
                 -- Determine if the current selected message was one of the
                 -- removed messages.
-                selMsgId <- use (csTeam(tId).tsMessageSelect.to selectMessageId)
+                selMsgId <- use (channelMessageSelect(cId).to selectMessageId)
                 let rmvdSel = do
                       i <- selMsgId -- :: Maybe MessageId
                       findMessage i removedMessages
@@ -389,8 +419,8 @@ addObtainedMessages cId reqCnt addTrailingGap posts = do
                       -- choice than allowing the user to perform select
                       -- actions on a message that isn't the one they just
                       -- selected.
-                      setMode Main
-                      csTeam(tId).tsMessageSelect .= MessageSelectState Nothing
+                      popMode tId
+                      channelMessageSelect(cId) .= MessageSelectState Nothing
 
                 -- Add a gap at each end of the newly fetched data, unless:
                 --   1. there is an overlap
@@ -405,7 +435,7 @@ addObtainedMessages cId reqCnt addTrailingGap posts = do
                     -- select to first (earliest) message)
                     case rmvdSelType of
                       Just (C UnknownGapBefore) ->
-                        csTeam(tId).tsMessageSelect .= MessageSelectState (pure $ MessagePostId earliestPId)
+                        channelMessageSelect(cId) .= MessageSelectState (pure $ MessagePostId earliestPId)
                       _ -> return ()
                   else do
                     -- add a gap at the start of the newly fetched block and
@@ -413,11 +443,11 @@ addObtainedMessages cId reqCnt addTrailingGap posts = do
                     -- the previously selected gap in this direction.
                     gapMsg <- newGapMessage (justBefore earliestDate) True
                     csChannels %= modifyChannelById cId
-                      (ccContents.cdMessages %~ addMessage gapMsg)
+                      (ccMessageInterface.miMessages %~ addMessage gapMsg)
                     -- Move selection from old gap to new gap
                     case rmvdSelType of
                       Just (C UnknownGapBefore) -> do
-                        csTeam(tId).tsMessageSelect .= MessageSelectState (gapMsg^.mMessageId)
+                        channelMessageSelect(cId) .= MessageSelectState (gapMsg^.mMessageId)
                       _ -> return ()
 
                 if reAddGapAfter
@@ -426,7 +456,7 @@ addObtainedMessages cId reqCnt addTrailingGap posts = do
                     -- select to last (latest) message.
                     case rmvdSelType of
                       Just (C UnknownGapAfter) ->
-                        csTeam(tId).tsMessageSelect .= MessageSelectState (pure $ MessagePostId latestPId)
+                        channelMessageSelect(cId) .= MessageSelectState (pure $ MessagePostId latestPId)
                       _ -> return ()
                   else do
                     -- add a gap at the end of the newly fetched block and
@@ -434,11 +464,11 @@ addObtainedMessages cId reqCnt addTrailingGap posts = do
                     -- the previously selected gap in this direction.
                     gapMsg <- newGapMessage (justAfter latestDate) False
                     csChannels %= modifyChannelById cId
-                      (ccContents.cdMessages %~ addMessage gapMsg)
+                      (ccMessageInterface.miMessages %~ addMessage gapMsg)
                     -- Move selection from old gap to new gap
                     case rmvdSelType of
                       Just (C UnknownGapAfter) ->
-                        csTeam(tId).tsMessageSelect .= MessageSelectState (gapMsg^.mMessageId)
+                        channelMessageSelect(cId) .= MessageSelectState (gapMsg^.mMessageId)
                       _ -> return ()
 
           case mTId of
@@ -500,6 +530,8 @@ addMessageToState doFetchMentionedUsers fetchAuthor newPostData = do
           OldPost p      -> (p, False)
           RecentPost p m -> (p, m)
 
+    hostname <- use (csResources.crConn.cdHostnameL)
+
     st <- use id
     case st ^? csChannel(postChannelId new) of
         Nothing -> do
@@ -536,7 +568,7 @@ addMessageToState doFetchMentionedUsers fetchAuthor newPostData = do
                 Nothing -> return Nothing
                 Just tId -> Just <$> getServerBaseUrl tId
 
-            let cp = toClientPost mBaseUrl new (new^.postRootIdL)
+            let cp = toClientPost hostname mBaseUrl new (new^.postRootIdL)
                 fromMe = (cp^.cpUser == (Just $ myUserId st)) &&
                          (isNothing $ cp^.cpUserOverride)
                 userPrefs = st^.csResources.crUserPreferences
@@ -548,6 +580,34 @@ addMessageToState doFetchMentionedUsers fetchAuthor newPostData = do
                   not (userPrefs^.userPrefShowJoinLeave) && isJoinOrLeave
                 cId = postChannelId new
 
+                -- TODO: match server-side logic to determine whether
+                -- this message counts as a post, for post-counting
+                -- purposes, to decide whether this should trigger an
+                -- increment of the total/viewed post counts
+                maybeIncrementTotalMessageCount =
+                    let shouldIncrement = case newPostData of
+                                              RecentPost {} ->
+                                                  True
+                                              _ ->
+                                                  False
+                    in if shouldIncrement
+                       then incrementTotalMessageCount
+                       else id
+
+                maybeIncrementViewedMessageCount currCId =
+                    let shouldIncrement = case newPostData of
+                                              RecentPost {} ->
+                                                  currCId == Just cId
+                                              _ ->
+                                                  False
+                    in if shouldIncrement
+                       then incrementViewedMessageCount
+                       else id
+
+                maybeIncrementMessageCounts currCId =
+                    maybeIncrementTotalMessageCount .
+                    maybeIncrementViewedMessageCount currCId
+
                 doAddMessage = do
                     -- Do we have the user data for the post author?
                     case cp^.cpUser of
@@ -557,8 +617,11 @@ addMessageToState doFetchMentionedUsers fetchAuthor newPostData = do
                             when (isNothing authorResult) $
                                 handleNewUsers (Seq.singleton authorId) (return ())
 
-                    curTId <- use csCurrentTeamId
-                    currCId <- use (csCurrentChannelId curTId)
+                    mcurTId <- use csCurrentTeamId
+                    currCId <- case mcurTId of
+                        Nothing -> return Nothing
+                        Just curTId -> use (csCurrentChannelId curTId)
+
                     flags <- use (csResources.crFlaggedPosts)
                     let (msg', mentionedUsers) = clientPostToMessage cp
                                  & _1.mFlagged .~ ((cp^.cpPostId) `Set.member` flags)
@@ -567,12 +630,13 @@ addMessageToState doFetchMentionedUsers fetchAuthor newPostData = do
                         fetchMentionedUsers mentionedUsers
 
                     csPostMap.at(postId new) .= Just msg'
-                    mh $ invalidateCacheEntry (ChannelMessages cId)
-                    mh $ invalidateCacheEntry $ RenderedMessage $ MessagePostId $ postId new
+                    invalidateChannelRenderingCache cId
+                    invalidateMessageRenderingCacheByPostId $ postId new
                     csChannels %= modifyChannelById cId
-                      ((ccContents.cdMessages %~ addMessage msg') .
+                      ((ccMessageInterface.miMessages %~ addMessage msg') .
                        (if not ignoredJoinLeaveMessage then adjustUpdated new else id) .
-                       (\c -> if currCId == cId
+                       maybeIncrementMessageCounts currCId .
+                       (\c -> if currCId == Just cId
                               then c
                               else case newPostData of
                                      OldPost _ -> c
@@ -582,8 +646,20 @@ addMessageToState doFetchMentionedUsers fetchAuthor newPostData = do
                               then c & ccInfo.cdMentionCount %~ succ
                               else c)
                       )
-                    asyncFetchReactionsForPost cId new
-                    asyncFetchAttachments new
+
+                    -- Check for whether the post is part of a thread
+                    -- being viewed. If so, add the post to that thread
+                    -- window as well.
+                    --
+                    -- Note that the team ID we pass here is either the
+                    -- team ID with which the channel is associated
+                    -- (mTId) or, if that is Nothing in the case of a DM
+                    -- channel, the currently-selected team (mcurTId)
+                    -- since all DM channels appear in all teams.
+                    addPostToOpenThread (mTId <|> mcurTId) new msg'
+
+                    updateSidebar mTId
+
                     postedChanMessage
 
                 doHandleAddedMessage = do
@@ -606,25 +682,51 @@ addMessageToState doFetchMentionedUsers fetchAuthor newPostData = do
 
                 postedChanMessage =
                   withChannelOrDefault (postChannelId new) NoAction $ \chan -> do
-                      currTid <- use csCurrentTeamId
-                      currCId <- use (csCurrentChannelId currTid)
+                      mcurrTid <- use csCurrentTeamId
+                      case mcurrTid of
+                          Nothing -> return NoAction
+                          Just currTid -> do
+                              currCId <- use (csCurrentChannelId currTid)
 
-                      let notifyPref = notifyPreference (myUser st) chan
-                          curChannelAction = if postChannelId new == currCId
-                                             then UpdateServerViewed
-                                             else NoAction
-                          originUserAction =
-                            if | fromMe                            -> NoAction
-                               | ignoredJoinLeaveMessage           -> NoAction
-                               | notifyPref == NotifyOptionAll     -> NotifyUser [newPostData]
-                               | notifyPref == NotifyOptionMention
-                                   && wasMentioned                 -> NotifyUser [newPostData]
-                               | otherwise                         -> NoAction
+                              let notifyPref = notifyPreference (myUser st) chan
+                                  curChannelAction = if Just (postChannelId new) == currCId
+                                                     then UpdateServerViewed
+                                                     else NoAction
+                                  originUserAction =
+                                    if | fromMe                            -> NoAction
+                                       | ignoredJoinLeaveMessage           -> NoAction
+                                       | notifyPref == NotifyOptionAll     -> NotifyUser [newPostData]
+                                       | notifyPref == NotifyOptionMention
+                                           && wasMentioned                 -> NotifyUser [newPostData]
+                                       | otherwise                         -> NoAction
 
-                      return $ curChannelAction `andProcessWith` originUserAction
+                              return $ curChannelAction `andProcessWith` originUserAction
 
             doHandleAddedMessage
 
+addPostToOpenThread :: Maybe TeamId -> Post -> Message -> MH ()
+addPostToOpenThread Nothing _ _ = return ()
+addPostToOpenThread (Just tId) new msg =
+    case postRootId new of
+        Nothing -> return ()
+        Just parentId -> do
+            mRoot <- preuse (maybeThreadInterface(tId)._Just.miRootPostId)
+            when (mRoot == Just parentId) $
+                modifyThreadMessages tId (new^.postChannelIdL) (addMessage msg)
+
+editPostInOpenThread :: Maybe TeamId -> Post -> Message -> MH ()
+editPostInOpenThread Nothing _ _ = return ()
+editPostInOpenThread (Just tId) new msg =
+     case postRootId new of
+        Nothing -> return ()
+        Just parentId -> do
+            mRoot <- preuse (maybeThreadInterface(tId)._Just.miRootPostId)
+            when (mRoot == Just parentId) $ do
+                mhLog LogGeneral "editPostInOpenThread: updating message"
+                let isEditedMessage m = m^.mMessageId == Just (MessagePostId $ new^.postIdL)
+                modifyEachThreadMessage tId (new^.postChannelIdL)
+                    (\m -> if isEditedMessage m then msg else m)
+
 -- | PostProcessMessageAdd is an internal value that informs the main
 -- code whether the user should be notified (e.g., ring the bell) or
 -- the server should be updated (e.g., that the channel has been
@@ -686,9 +788,6 @@ data PostToAdd =
     -- only available in websocket events (and then provided to this
     -- constructor).
 
-encodeToJSONstring :: A.ToJSON a => a -> String
-encodeToJSONstring a = BL8.unpack $ A.encode a
-
 -- Notification Version 2 payload definition
 data NotificationV2 = NotificationV2
     { version :: Int
@@ -696,6 +795,7 @@ data NotificationV2 = NotificationV2
     , mention :: Bool
     , from :: Text
     } deriving (Show)
+
 instance A.ToJSON NotificationV2 where
     toJSON (NotificationV2 vers msg mentioned sender) =
         A.object [ "version"  A..= vers
@@ -704,15 +804,50 @@ instance A.ToJSON NotificationV2 where
                  , "from"     A..= sender
                  ]
 
+-- Notification Version 3 payload definition
+data NotificationV3 = NotificationV3
+    { notifyV3Version :: Int
+    , notifyV3Message :: Text
+    , notifyV3Mention :: Bool
+    , notifyV3From :: Text
+    , notifyV3MessageType :: Text
+    } deriving (Show)
+
+instance A.ToJSON NotificationV3 where
+    toJSON (NotificationV3 vers msg mentioned sender msgTy) =
+        A.object [ "version"     A..= vers
+                 , "message"     A..= msg
+                 , "mention"     A..= mentioned
+                 , "from"        A..= sender
+                 , "messageType" A..= msgTy
+                 ]
+
 -- We define a notifyGetPayload for each notification version.
-notifyGetPayload :: NotificationVersion -> ChatState -> Post -> Bool -> Maybe String
-notifyGetPayload NotifyV1 _ _ _ = do return ""
+notifyGetPayload :: NotificationVersion -> ChatState -> Post -> Bool -> Maybe BSL.ByteString
+notifyGetPayload NotifyV1 _ _ _ = do return mempty
 notifyGetPayload NotifyV2 st post mentioned = do
     let notification = NotificationV2 2 msg mentioned sender
-    return (encodeToJSONstring notification)
+    return (A.encode notification)
         where
             msg = sanitizeUserText $ postMessage post
             sender = maybePostUsername st post
+notifyGetPayload NotifyV3 st post mentioned = do
+    let notification = NotificationV3 3 msg mentioned sender msgTy
+    return (A.encode notification)
+        where
+            msg = sanitizeUserText $ postMessage post
+            sender = maybePostUsername st post
+            msgTy = case postType post of
+                PostTypeJoinChannel       -> "joinChannel"
+                PostTypeLeaveChannel      -> "leaveChannel"
+                PostTypeAddToChannel      -> "addToChannel"
+                PostTypeRemoveFromChannel -> "removeFromChannel"
+                PostTypeHeaderChange      -> "headerChange"
+                PostTypeDisplayNameChange -> "displayNameChange"
+                PostTypePurposeChange     -> "purposeChange"
+                PostTypeChannelDeleted    -> "channelDeleted"
+                PostTypeEphemeral         -> "ephemeral"
+                PostTypeUnknown _         -> "unknown"
 
 handleNotifyCommand :: Post -> Bool -> NotificationVersion -> MH ()
 handleNotifyCommand post mentioned NotifyV1 = do
@@ -740,13 +875,22 @@ handleNotifyCommand post mentioned NotifyV2 = do
             doAsyncWith Preempt $ do
                 runLoggedCommand outputChan (T.unpack cmd) [] payload Nothing
                 return Nothing
+handleNotifyCommand post mentioned NotifyV3 = do
+    outputChan <- use (csResources.crSubprocessLog)
+    st <- use id
+    let payload = notifyGetPayload NotifyV3 st post mentioned
+    notifyCommand <- use (csResources.crConfiguration.configActivityNotifyCommandL)
+    case notifyCommand of
+        Nothing -> return ()
+        Just cmd ->
+            doAsyncWith Preempt $ do
+                runLoggedCommand outputChan (T.unpack cmd) [] payload Nothing
+                return Nothing
 
 runNotifyCommand :: Post -> Bool -> MH ()
 runNotifyCommand post mentioned = do
     notifyVersion <- use (csResources.crConfiguration.configActivityNotifyVersionL)
-    case notifyVersion of
-        NotifyV1 -> handleNotifyCommand post mentioned NotifyV1
-        NotifyV2 -> handleNotifyCommand post mentioned NotifyV2
+    handleNotifyCommand post mentioned notifyVersion
 
 maybePostUsername :: ChatState -> Post -> T.Text
 maybePostUsername st p =
@@ -788,27 +932,25 @@ maybePostUsername st p =
 -- messages to use as a starting point, but exceptions are new
 -- channels and empty channels.
 asyncFetchMoreMessages :: MH ()
-asyncFetchMoreMessages = do
-    tId <- use csCurrentTeamId
-    cId <- use (csCurrentChannelId tId)
-    withChannel cId $ \chan ->
-        let offset = max 0 $ length (chan^.ccContents.cdMessages) - 2
-            page = offset `div` pageAmount
-            usefulMsgs = getTwoContiguousPosts Nothing (chan^.ccContents.cdMessages.to reverseMessages)
-            sndOldestId = (messagePostId . snd) =<< usefulMsgs
-            query = MM.defaultPostQuery
-                      { MM.postQueryPage = maybe (Just page) (const Nothing) sndOldestId
-                      , MM.postQueryPerPage = Just pageAmount
-                      , MM.postQueryBefore = sndOldestId
-                      }
-            addTrailingGap = MM.postQueryBefore query == Nothing &&
-                             MM.postQueryPage query == Just 0
-        in doAsyncChannelMM Preempt cId
-               (\s c -> MM.mmGetPostsForChannel c query s)
-               (\c p -> Just $ do
-                   pp <- addObtainedMessages c (-pageAmount) addTrailingGap p
-                   postProcessMessageAdd pp)
-
+asyncFetchMoreMessages =
+    withCurrentTeam $ \tId ->
+        withCurrentChannel tId $ \cId chan -> do
+            let offset = max 0 $ length (chan^.ccMessageInterface.miMessages) - 2
+                page = offset `div` pageAmount
+                usefulMsgs = getTwoContiguousPosts Nothing (chan^.ccMessageInterface.miMessages.to reverseMessages)
+                sndOldestId = (messagePostId . snd) =<< usefulMsgs
+                query = MM.defaultPostQuery
+                          { MM.postQueryPage = maybe (Just page) (const Nothing) sndOldestId
+                          , MM.postQueryPerPage = Just pageAmount
+                          , MM.postQueryBefore = sndOldestId
+                          }
+                addTrailingGap = MM.postQueryBefore query == Nothing &&
+                                 MM.postQueryPage query == Just 0
+            doAsyncChannelMM Preempt cId
+                (\s c -> MM.mmGetPostsForChannel c query s)
+                (\c p -> Just $ do
+                    pp <- addObtainedMessages c (-pageAmount) addTrailingGap p
+                    postProcessMessageAdd pp)
 
 -- | Given a starting point and a direction to move from that point,
 -- returns the closest two adjacent messages on that direction (as a
@@ -832,9 +974,9 @@ asyncFetchMessagesForGap :: ChannelId -> Message -> MH ()
 asyncFetchMessagesForGap cId gapMessage =
   when (isGap gapMessage) $
   withChannel cId $ \chan ->
-    let offset = max 0 $ length (chan^.ccContents.cdMessages) - 2
+    let offset = max 0 $ length (chan^.ccMessageInterface.miMessages) - 2
         page = offset `div` pageAmount
-        chanMsgs = chan^.ccContents.cdMessages
+        chanMsgs = chan^.ccMessageInterface.miMessages
         fromMsg = Just gapMessage
         fetchNewer = case gapMessage^.mType of
                        C UnknownGapAfter -> True
@@ -906,14 +1048,12 @@ asyncFetchMessagesSurrounding cId pId = do
               let pl = toList $ postsOrder posts
               in if length pl > 1 then Just $ last $ init pl else Nothing
 
-fetchVisibleIfNeeded :: MH ()
-fetchVisibleIfNeeded = do
+fetchVisibleIfNeeded :: TeamId -> MH ()
+fetchVisibleIfNeeded tId = do
     sts <- use csConnectionStatus
     when (sts == Connected) $ do
-        tId <- use csCurrentTeamId
-        cId <- use (csCurrentChannelId tId)
-        withChannel cId $ \chan -> do
-            let msgs = chan^.ccContents.cdMessages.to reverseMessages
+        withCurrentChannel tId $ \cId chan -> do
+            let msgs = chan^.ccMessageInterface.miMessages.to reverseMessages
                 (numRemaining, gapInDisplayable, _, rel'pId, overlap) =
                     foldl gapTrail (numScrollbackPosts, False, Nothing, Nothing, 2) msgs
 
@@ -938,39 +1078,12 @@ fetchVisibleIfNeeded = do
                 op = \s c -> MM.mmGetPostsForChannel c finalQuery s
                 addTrailingGap = MM.postQueryBefore finalQuery == Nothing &&
                                  MM.postQueryPage finalQuery == Just 0
-
-            when ((not $ chan^.ccContents.cdFetchPending) && gapInDisplayable) $ do
-                csChannel(cId).ccContents.cdFetchPending .= True
-                doAsyncChannelMM Preempt cId op
-                    (\c p -> Just $ do
-                        csChannel(c).ccContents.cdFetchPending .= False
-                        addObtainedMessages c (-numToRequest) addTrailingGap p >>= postProcessMessageAdd)
-
-asyncFetchAttachments :: Post -> MH ()
-asyncFetchAttachments p = do
-    let cId = p^.postChannelIdL
-        pId = p^.postIdL
-    session <- getSession
-    host <- use (csResources.crConn.cdHostnameL)
-    F.forM_ (p^.postFileIdsL) $ \fId -> doAsyncWith Normal $ do
-        info <- MM.mmGetMetadataForFile fId session
-        let scheme = "https://"
-            attUrl = scheme <> host <> urlForFile fId
-            attachment = mkAttachment (fileInfoName info) attUrl fId
-            addIfMissing a as =
-                if isNothing $ Seq.elemIndexL a as
-                then a Seq.<| as
-                else as
-            addAttachment m
-                | m^.mMessageId == Just (MessagePostId pId) =
-                    m & mAttachments %~ (addIfMissing attachment)
-                | otherwise =
-                    m
-        return $ Just $ do
-            csChannel(cId).ccContents.cdMessages.traversed %= addAttachment
-            mh $ do
-                invalidateCacheEntry $ ChannelMessages cId
-                invalidateCacheEntry $ RenderedMessage $ MessagePostId pId
+            when ((not $ chan^.ccInfo.cdFetchPending) && gapInDisplayable) $ do
+                   csChannel(cId).ccInfo.cdFetchPending .= True
+                   doAsyncChannelMM Preempt cId op
+                       (\c p -> Just $ do
+                           csChannel(c).ccInfo.cdFetchPending .= False
+                           addObtainedMessages c (-numToRequest) addTrailingGap p >>= postProcessMessageAdd)
 
 -- | Given a post ID, switch to that post's channel and select the post
 -- in message selection mode.
@@ -981,7 +1094,7 @@ asyncFetchAttachments p = do
 -- reasonable effort will be made (fetch the post, join the channel)
 -- before giving up.
 jumpToPost :: PostId -> MH ()
-jumpToPost pId = do
+jumpToPost pId = withCurrentTeam $ \tId -> do
     st <- use id
     case getMessageForPostId st pId of
       Just msg ->
@@ -990,11 +1103,11 @@ jumpToPost pId = do
               -- Are we a member of the channel?
               case findChannelById cId (st^.csChannels) of
                   Nothing ->
-                      joinChannel' cId (Just $ jumpToPost pId)
+                      joinChannel' tId cId (Just $ jumpToPost pId)
                   Just _ -> do
-                      setFocus cId
-                      setMode MessageSelect
-                      csCurrentTeam.tsMessageSelect .= MessageSelectState (msg^.mMessageId)
+                      setFocus tId cId
+                      beginMessageSelect (csChannelMessageInterface(cId))
+                      channelMessageSelect(cId) .= MessageSelectState (msg^.mMessageId)
           Nothing ->
             error "INTERNAL: selected Post ID not associated with a channel"
       Nothing -> do
@@ -1009,7 +1122,7 @@ jumpToPost pId = do
                               -- If not, join it and then try jumping to
                               -- the post if the channel join is successful.
                               Nothing -> do
-                                  joinChannel' (postChannelId p) (Just $ jumpToPost pId)
+                                  joinChannel' tId (postChannelId p) (Just $ jumpToPost pId)
                               -- Otherwise add the post to the state and
                               -- then jump.
                               Just _ -> do
