@@ -10,7 +10,6 @@ where
 import           Prelude ()
 import           Matterhorn.Prelude
 
-import           Brick.Main ( invalidateCacheEntry )
 import qualified Data.Map.Strict as Map
 import           Lens.Micro.Platform
 import qualified Data.Set as S
@@ -20,7 +19,7 @@ import           Network.Mattermost.Lenses
 import           Network.Mattermost.Types
 
 import           Matterhorn.State.Async
-import           Matterhorn.State.Common ( fetchMentionedUsers )
+import           Matterhorn.State.Common
 import           Matterhorn.Types
 
 
@@ -41,8 +40,15 @@ asyncFetchReactionsForPost cId p
 -- incoming reactions.
 addReactions :: ChannelId -> [Reaction] -> MH ()
 addReactions cId rs = do
-    mh $ invalidateCacheEntry $ ChannelMessages cId
-    csChannel(cId).ccContents.cdMessages %= fmap upd
+    invalidateChannelRenderingCache cId
+    csChannelMessages(cId) %= fmap upd
+
+    -- Also update any open thread for the corresponding channel's team
+    withChannel cId $ \chan -> do
+        case chan^.ccInfo.cdTeamId of
+            Nothing -> return ()
+            Just tId -> modifyThreadMessages tId cId (fmap upd)
+
     let mentions = S.fromList $ UserIdMention <$> reactionUserId <$> rs
     fetchMentionedUsers mentions
     invalidateRenderCache
@@ -53,9 +59,8 @@ addReactions cId rs = do
           | otherwise = id
         insertAll mId msg = foldr (insert mId) msg rs
         invalidateRenderCache = do
-          let cacheIds = map cacheIdOf rs
-          mh $ mapM_ invalidateCacheEntry cacheIds
-        cacheIdOf r = RenderedMessage $ MessagePostId (r^.reactionPostIdL)
+            forM_ rs $ \r ->
+                invalidateMessageRenderingCacheByPostId $ r^.reactionPostIdL
 
 -- | Remove the specified reaction from its message in the specified
 -- channel. This should only be called in response to a server event
@@ -64,15 +69,23 @@ addReactions cId rs = do
 -- rendered message corresponding to the removed reaction.
 removeReaction :: Reaction -> ChannelId -> MH ()
 removeReaction r cId = do
-    mh $ invalidateCacheEntry $ ChannelMessages cId
-    csChannel(cId).ccContents.cdMessages %= fmap upd
+    invalidateChannelRenderingCache cId
+    csChannelMessages(cId) %= fmap upd
+
+    -- Also update any open thread for the corresponding channel's team
+    withChannel cId $ \chan -> do
+        case chan^.ccInfo.cdTeamId of
+            Nothing -> return ()
+            Just tId -> modifyThreadMessages tId cId (fmap upd)
+
     invalidateRenderCache
   where upd m | m^.mMessageId == Just (MessagePostId $ r^.reactionPostIdL) =
-                  m & mReactions %~ (Map.alter delReaction (r^.reactionEmojiNameL))
+                  m & mReactions %~ (removeEmptySets . Map.alter delReaction (r^.reactionEmojiNameL))
               | otherwise = m
         delReaction mUs = S.delete (r^.reactionUserIdL) <$> mUs
+        removeEmptySets = Map.filter (not . S.null)
         invalidateRenderCache =
-          mh $ invalidateCacheEntry $ RenderedMessage $ MessagePostId (r^.reactionPostIdL)
+            invalidateMessageRenderingCacheByPostId $ r^.reactionPostIdL
 
 -- | Set or unset a reaction on a post.
 updateReaction :: PostId -> Text -> Bool -> MH ()
@@ -92,4 +105,4 @@ toggleReaction :: PostId -> Text -> Set UserId -> MH ()
 toggleReaction pId text uIds = do
     myId <- gets myUserId
     let current = myId `S.member` uIds
-    updateReaction pId text (not current)
\ No newline at end of file
+    updateReaction pId text (not current)
