@@ -37,6 +37,7 @@ module Matterhorn.Themes
   , errorMessageAttr
   , helpAttr
   , helpEmphAttr
+  , helpKeyEventAttr
   , channelSelectPromptAttr
   , channelSelectMatchAttr
   , completionAlternativeListAttr
@@ -58,6 +59,8 @@ module Matterhorn.Themes
   , tabUnselectedAttr
   , buttonAttr
   , buttonFocusedAttr
+  , threadAttr
+  , focusedEditorPromptAttr
 
   -- * Username formatting
   , colorUsername
@@ -89,154 +92,163 @@ import           Matterhorn.Types ( InternalTheme(..), specialUserMentions )
 
 
 helpAttr :: AttrName
-helpAttr = "help"
+helpAttr = attrName "help"
 
 helpEmphAttr :: AttrName
-helpEmphAttr = "helpEmphasis"
+helpEmphAttr = attrName "helpEmphasis"
+
+helpKeyEventAttr :: AttrName
+helpKeyEventAttr = attrName "helpKeyEvent"
 
 recentMarkerAttr :: AttrName
-recentMarkerAttr = "recentChannelMarker"
+recentMarkerAttr = attrName "recentChannelMarker"
 
 replyParentAttr :: AttrName
-replyParentAttr = "replyParentPreview"
+replyParentAttr = attrName "replyParentPreview"
 
 pinnedMessageIndicatorAttr :: AttrName
-pinnedMessageIndicatorAttr = "pinnedMessageIndicator"
+pinnedMessageIndicatorAttr = attrName "pinnedMessageIndicator"
 
 loadMoreAttr :: AttrName
-loadMoreAttr = "loadMoreMessages"
+loadMoreAttr = attrName "loadMoreMessages"
 
 urlListSelectedAttr :: AttrName
-urlListSelectedAttr = "urlListCursor"
+urlListSelectedAttr = attrName "urlListCursor"
 
 messageSelectAttr :: AttrName
-messageSelectAttr = "messageSelectCursor"
+messageSelectAttr = attrName "messageSelectCursor"
 
 editedMarkingAttr :: AttrName
-editedMarkingAttr = "editedMarking"
+editedMarkingAttr = attrName "editedMarking"
 
 editedRecentlyMarkingAttr :: AttrName
-editedRecentlyMarkingAttr = "editedRecentlyMarking"
+editedRecentlyMarkingAttr = attrName "editedRecentlyMarking"
 
 permalinkAttr :: AttrName
-permalinkAttr = "permalink"
+permalinkAttr = attrName "permalink"
 
 dialogAttr :: AttrName
-dialogAttr = "dialog"
+dialogAttr = attrName "dialog"
 
 dialogEmphAttr :: AttrName
-dialogEmphAttr = "dialogEmphasis"
+dialogEmphAttr = attrName "dialogEmphasis"
 
 channelSelectMatchAttr :: AttrName
-channelSelectMatchAttr = "channelSelectMatch"
+channelSelectMatchAttr = attrName "channelSelectMatch"
 
 channelSelectPromptAttr :: AttrName
-channelSelectPromptAttr = "channelSelectPrompt"
+channelSelectPromptAttr = attrName "channelSelectPrompt"
 
 completionAlternativeListAttr :: AttrName
-completionAlternativeListAttr = "tabCompletionAlternative"
+completionAlternativeListAttr = attrName "tabCompletionAlternative"
 
 completionAlternativeCurrentAttr :: AttrName
-completionAlternativeCurrentAttr = "tabCompletionCursor"
+completionAlternativeCurrentAttr = attrName "tabCompletionCursor"
 
 timeAttr :: AttrName
-timeAttr = "time"
+timeAttr = attrName "time"
 
 currentUserAttr :: AttrName
-currentUserAttr = "currentUser"
+currentUserAttr = attrName "currentUser"
 
 channelHeaderAttr :: AttrName
-channelHeaderAttr = "channelHeader"
+channelHeaderAttr = attrName "channelHeader"
 
 verbatimTruncateMessageAttr :: AttrName
-verbatimTruncateMessageAttr = "verbatimTruncateMessage"
+verbatimTruncateMessageAttr = attrName "verbatimTruncateMessage"
 
 channelListHeaderAttr :: AttrName
-channelListHeaderAttr = "channelListSectionHeader"
+channelListHeaderAttr = attrName "channelListSectionHeader"
 
 currentChannelNameAttr :: AttrName
-currentChannelNameAttr = "currentChannelName"
+currentChannelNameAttr = attrName "currentChannelName"
 
 channelNameAttr :: AttrName
-channelNameAttr = "channelName"
+channelNameAttr = attrName "channelName"
 
 unreadChannelAttr :: AttrName
-unreadChannelAttr = "unreadChannel"
+unreadChannelAttr = attrName "unreadChannel"
 
 unreadGroupMarkerAttr :: AttrName
-unreadGroupMarkerAttr = "unreadChannelGroupMarker"
+unreadGroupMarkerAttr = attrName "unreadChannelGroupMarker"
 
 mentionsChannelAttr :: AttrName
-mentionsChannelAttr = "channelWithMentions"
+mentionsChannelAttr = attrName "channelWithMentions"
 
 currentTeamAttr :: AttrName
-currentTeamAttr = "currentTeam"
+currentTeamAttr = attrName "currentTeam"
 
 tabSelectedAttr :: AttrName
-tabSelectedAttr = "tabSelected"
+tabSelectedAttr = attrName "tabSelected"
 
 tabUnselectedAttr :: AttrName
-tabUnselectedAttr = "tabUnselected"
+tabUnselectedAttr = attrName "tabUnselected"
 
 dateTransitionAttr :: AttrName
-dateTransitionAttr = "dateTransition"
+dateTransitionAttr = attrName "dateTransition"
 
 newMessageTransitionAttr :: AttrName
-newMessageTransitionAttr = "newMessageTransition"
+newMessageTransitionAttr = attrName "newMessageTransition"
 
 urlAttr :: AttrName
-urlAttr = "url"
+urlAttr = attrName "url"
 
 codeAttr :: AttrName
-codeAttr = "codeBlock"
+codeAttr = attrName "codeBlock"
 
 emailAttr :: AttrName
-emailAttr = "email"
+emailAttr = attrName "email"
 
 emojiAttr :: AttrName
-emojiAttr = "emoji"
+emojiAttr = attrName "emoji"
 
 reactionAttr :: AttrName
-reactionAttr = "reaction"
+reactionAttr = attrName "reaction"
 
 myReactionAttr :: AttrName
-myReactionAttr = reactionAttr <> "mine"
+myReactionAttr = reactionAttr <> attrName "mine"
 
 clientMessageAttr :: AttrName
-clientMessageAttr = "clientMessage"
+clientMessageAttr = attrName "clientMessage"
 
 clientHeaderAttr :: AttrName
-clientHeaderAttr = "markdownHeader"
+clientHeaderAttr = attrName "markdownHeader"
 
 strikeThroughAttr :: AttrName
-strikeThroughAttr = "markdownStrikethrough"
+strikeThroughAttr = attrName "markdownStrikethrough"
 
 clientEmphAttr :: AttrName
-clientEmphAttr = "markdownEmph"
+clientEmphAttr = attrName "markdownEmph"
 
 clientStrongAttr :: AttrName
-clientStrongAttr = "markdownStrong"
+clientStrongAttr = attrName "markdownStrong"
 
 errorMessageAttr :: AttrName
-errorMessageAttr = "errorMessage"
+errorMessageAttr = attrName "errorMessage"
 
 gapMessageAttr :: AttrName
-gapMessageAttr = "gapMessage"
+gapMessageAttr = attrName "gapMessage"
 
 misspellingAttr :: AttrName
-misspellingAttr = "misspelling"
+misspellingAttr = attrName "misspelling"
 
 messageSelectStatusAttr :: AttrName
-messageSelectStatusAttr = "messageSelectStatus"
+messageSelectStatusAttr = attrName "messageSelectStatus"
 
 urlSelectStatusAttr :: AttrName
-urlSelectStatusAttr = "urlSelectStatus"
+urlSelectStatusAttr = attrName "urlSelectStatus"
 
 buttonAttr :: AttrName
-buttonAttr = "button"
+buttonAttr = attrName "button"
 
 buttonFocusedAttr :: AttrName
-buttonFocusedAttr = buttonAttr <> "focused"
+buttonFocusedAttr = buttonAttr <> attrName "focused"
+
+threadAttr :: AttrName
+threadAttr = attrName "thread"
+
+focusedEditorPromptAttr :: AttrName
+focusedEditorPromptAttr = attrName "focusedEditorPrompt"
 
 lookupTheme :: Text -> Maybe InternalTheme
 lookupTheme n = find ((== n) . internalThemeName) internalThemes
@@ -287,9 +299,14 @@ lightAttrs usernameColors =
     in [ (timeAttr,                         fg black)
        , (buttonAttr,                       black `on` cyan)
        , (buttonFocusedAttr,                black `on` yellow)
+       , (threadAttr,                       defAttr)
+       , (focusedEditorPromptAttr,          fg yellow `withStyle` bold)
        , (currentUserAttr,                  defAttr `withStyle` bold)
        , (channelHeaderAttr,                fg black)
        , (verbatimTruncateMessageAttr,      fg blue)
+       , (scrollbarAttr,                    defAttr)
+       , (scrollbarHandleAttr,              defAttr `withStyle` reverseVideo)
+       , (scrollbarTroughAttr,              defAttr)
        , (channelListHeaderAttr,            fg cyan)
        , (currentChannelNameAttr,           black `on` yellow `withStyle` bold)
        , (unreadChannelAttr,                black `on` cyan   `withStyle` bold)
@@ -314,6 +331,7 @@ lightAttrs usernameColors =
        , (helpAttr,                         fg black)
        , (pinnedMessageIndicatorAttr,       black `on` cyan)
        , (helpEmphAttr,                     fg blue `withStyle` bold)
+       , (helpKeyEventAttr,                 fg magenta)
        , (channelSelectMatchAttr,           black `on` magenta `withStyle` underline)
        , (channelSelectPromptAttr,          fg black)
        , (completionAlternativeListAttr,    white `on` blue)
@@ -352,6 +370,8 @@ darkAttrs usernameColors =
   in [ (timeAttr,                         fg white)
      , (buttonAttr,                       black `on` cyan)
      , (buttonFocusedAttr,                black `on` yellow)
+     , (threadAttr,                       defAttr)
+     , (focusedEditorPromptAttr,          fg yellow `withStyle` bold)
      , (currentUserAttr,                  defAttr `withStyle` bold)
      , (channelHeaderAttr,                fg white)
      , (verbatimTruncateMessageAttr,      fg cyan)
@@ -360,6 +380,9 @@ darkAttrs usernameColors =
      , (unreadChannelAttr,                black `on` cyan   `withStyle` bold)
      , (unreadGroupMarkerAttr,            fg white `withStyle` bold)
      , (mentionsChannelAttr,              black `on` brightMagenta `withStyle` bold)
+     , (scrollbarAttr,                    defAttr)
+     , (scrollbarHandleAttr,              defAttr `withStyle` reverseVideo)
+     , (scrollbarTroughAttr,              defAttr)
      , (urlAttr,                          fg yellow)
      , (emailAttr,                        fg yellow)
      , (codeAttr,                         fg magenta)
@@ -379,6 +402,7 @@ darkAttrs usernameColors =
      , (gapMessageAttr,                   black `on` yellow)
      , (helpAttr,                         fg white)
      , (helpEmphAttr,                     fg cyan `withStyle` bold)
+     , (helpKeyEventAttr,                 fg yellow)
      , (channelSelectMatchAttr,           black `on` magenta `withStyle` underline)
      , (channelSelectPromptAttr,          fg white)
      , (completionAlternativeListAttr,    white `on` blue)
@@ -431,7 +455,7 @@ darkColor256Theme = InternalTheme name theme desc
         desc = "Like builtin:dark, but with 256-color username colors"
 
 usernameAttr :: Int -> AttrName
-usernameAttr i = "username" <> (attrName $ show i)
+usernameAttr i = attrName "username" <> (attrName $ show i)
 
 -- | Render a string with a color chosen based on the text of a
 -- username.
@@ -650,6 +674,9 @@ themeDocs = ThemeDocumentation $ M.fromList $
     , ( helpEmphAttr
       , "The help screen's emphasized text"
       )
+    , ( helpKeyEventAttr
+      , "The help screen's mention of key event names"
+      )
     , ( channelSelectPromptAttr
       , "Channel selection: prompt"
       )
@@ -848,6 +875,21 @@ themeDocs = ThemeDocumentation $ M.fromList $
     , ( verbatimTruncateMessageAttr
       , "Attribute for a message indicating that a verbatim or code block has been only partially displayed"
       )
+    , ( scrollbarAttr
+      , "Base aAttribute for scroll bars"
+      )
+    , ( scrollbarTroughAttr
+      , "Attribute for scroll bar troughs"
+      )
+    , ( scrollbarHandleAttr
+      , "Attribute for scroll bar handles"
+      )
+    , ( threadAttr
+      , "Base attribute for the thread window"
+      )
+    , ( focusedEditorPromptAttr
+      , "The attribute for the prompt of the focused message editor"
+      )
     ] <> [ (usernameAttr i, T.pack $ "Username color " <> show i)
          | i <- [0..usernameColorHashBuckets-1]
          ]
