@@ -11,6 +11,7 @@
 module Matterhorn.Types.RichText
   ( Blocks(..)
   , unBlocks
+  , singleB
 
   , Block(..)
   , sameBlockType
@@ -18,6 +19,7 @@ module Matterhorn.Types.RichText
   , Inline(..)
   , Inlines(..)
   , unInlines
+  , singleI
 
   , C.ListType(..)
   , C.ListSpacing(..)
@@ -71,7 +73,7 @@ data TeamBaseURL = TeamBaseURL TeamURLName ServerBaseURL
 
 -- | A sequence of rich text blocks.
 newtype Blocks = Blocks (Seq Block)
-            deriving (Semigroup, Monoid, Show)
+            deriving (Semigroup, Monoid, Show, Eq)
 
 unBlocks :: Blocks -> Seq Block
 unBlocks (Blocks bs) = bs
@@ -100,7 +102,7 @@ data Block =
     -- ^ A horizontal rule.
     | Table [C.ColAlignment] [Inlines] [[Inlines]]
     -- ^ A table.
-    deriving (Show)
+    deriving (Show, Eq)
 
 -- | Returns whether two blocks have the same type.
 sameBlockType :: Block -> Block -> Bool
@@ -267,8 +269,15 @@ parseUsername :: (Monad m) => C.InlineParser m Inlines
 parseUsername = P.try $ do
     void $ C.symbol userSigilChar
     let chunk = C.satisfyWord (const True) <|> C.symbol '_' <|> C.symbol '-'
-        [period] = C.tokenize "" "."
-    uts <- intersperse period <$> P.sepBy1 chunk (C.symbol '.')
+        period = case C.tokenize "" "." of
+            [p] -> p
+            _ -> error "BUG: parseUsername: failed to tokenize basic input"
+    uts <- intersperse period <$> do
+        c <- chunk
+        rest <- P.many $ P.try $ do
+            void $ C.symbol '.'
+            chunk
+        return $ c : rest
     return $ singleI $ EUser $ C.untokenize uts
 
 -- Syntax extension for parsing :emoji: references.
