@@ -3,7 +3,7 @@ module Matterhorn.Zipper
   , fromList
   , toList
   , focus
-  , unsafeFocus
+  , focusHeading
   , left
   , leftL
   , right
@@ -75,8 +75,14 @@ rightL = lens right (\ _ b -> left b)
 focus :: Zipper a b -> Maybe b
 focus = C.focus . zRing
 
-unsafeFocus :: Zipper a b -> b
-unsafeFocus = fromJust . focus
+-- Return the heading value corresponding to the focused value, if any
+-- element is in focus
+focusHeading :: (Eq b) => Zipper a b -> Maybe a
+focusHeading z = do
+    f <- focus z
+    let matchesElems (_, es) = f `elem` es
+        matches = filter matchesElems $ toList z
+    listToMaybe $ fst <$> matches
 
 -- Turn a list into a wraparound zipper, focusing on the head
 fromList :: (Eq b) => [(a, [b])] -> Zipper a b
